/******************************************************/
/* TITLE: AUTOCORR                                    */
/*                                                    */
/* PURPOSE: CALCULATES THE AUTOCORRELATION FUNCTION   */
/* OF A DATA SET OF LENGTH n AND RETURNS LAGS 0 THRU  */
/*          l IN R                                    */
/*                                                    */
/* USAGE: R = autocorr(data,n,l);                     */
/*                                                    */
/* ARGUMENTS:                                         */
/* data : INPUT VECTOR OF LENGTH n CONTAINING THE     */
/*        TIME SERIES (TYPE *double)                  */
/* n    : LENGTH OF data (TYPE int)                   */
/* l    : LENGTH OF R, l < r IF NOT R[0] IS SET EQUAL */
/*        TO ZERO AND RETURN IS MADE (TYPE int)       */
/*                                                    */
/* REMARK:  databar = (1/n)*(SUM:i=0,n|data[i])       */
/* R[j] = (SUM:i=0,n-j | (data[i] - databar)*         */
/*        (data[i+j] - databar)) /                    */
/*    sqrt(SUM:i=0,n-j |((data[i] - databar))^2*      */
/*         SUM:i=0,n-j |((data[i+j] - databar))^2)    */
/******************************************************/

#include <stdio.h>
#include <math.h>
#include <time.h>
#include <stdlib.h>

double *autocorr(double *data,int n,int l);

double *autocorr(double *data,int n,int l)
{
  double *R,aver,sum, sum1, sum2;
  int i,j,ij,nj;
  
  R = ( double* ) malloc ( (unsigned) l*sizeof(double) );

  if(l >= n)
    {
      R[0] = 0;
      return R;
    }

/*Average (Mean)*/
  aver = 0;
  for(i=0; i<n; i++)
    {
	  //printf("data[%d] = %.8g \n",i,data[i]);
      aver += data[i];
    }
  aver = aver/n;

/*lag index loop*/ 
  for(j=0; j<l; j++)
    {
      nj = n-j;
      sum = 0; sum1 = 0; sum2 = 0;

/*AutoCovariance, AutoVariance1 and AutoVariance2*/ 
      for(i=0; i<nj; i++)
			{
			ij = i+j;
			sum += (data[i]-aver)*(data[ij]-aver);
			sum1 += (data[i]-aver)*(data[i]-aver);
			sum2 += (data[ij]-aver)*(data[ij]-aver);
			}

/*AutoCorrelation*/
	R[j] = sum/sqrt(sum1*sum2);

    }
  
  return R;
}
