// DSGE.c : Defines the entry point for the console application.
#include "stdafx.h"


//Declaring and initializing a global seed:
//(see "Storage-Class Specifiers for External-Level Declarations")
gsl_rng * _gsl_r = gsl_rng_alloc (gsl_rng_taus); //Definition and intialization of _gsl_r

//Declaring and initializing a global derivative step size:
extern double h; //Reference to global derivative step size h defined on the next line
double h = 0.0001; //Definition and intialization of h


// Matrix form conversion function from MKL to GSL of size (GSL_row x GSL_col)
void MKLtoGSL (double MKL_mat[], gsl_matrix * GSL_mat, int GSL_row, int GSL_col);

// Matrix form conversion function from GSL of size (GSL_row x GSL_col) to MKL
void GSLtoMKL (gsl_matrix * GSL_mat, double MKL_mat[], int GSL_row, int GSL_col);

// Function for printing GSL matrix to screen
void print_scr_gsl_mat (gsl_matrix * GSL_mat, int r, int c);

// Function for printing a GSL vector to screen
void print_scr_gsl_vec (gsl_vector * GSL_vec, int c);

// Function for printing MKL matrix to screen
void print_scr_mkl_mat (double MKL_mat[], int r, int c);

// Function for setting the first n elements of an array a equal to the first n elements of array b
void eqarrays(double a[], double b[], int n);

//MLE procedures:
void SimAnneal(dsge_param *THETA_par, gsl_vector *v_THETA_MLE_par, dsge_data_obj *data_X, int data_T);
void BFGS(DSGE_like_params *param, gsl_vector *v_THETA_MLE_par);

//M-H procedures
void MH_Tailoring(DSGE_like_params * param, dsge_param *THETA, gsl_vector *v_THETA_MLE, int MH_T, unsigned int Tailor);

void DSGE_like_df (const gsl_vector *v_THETA, void *p, gsl_vector *gradient);

//Likelihood function (used for testing)
double lglike_fn(dsge_param *THETA_par, dsge_data_obj *data_X_par, int MH_i_par, int data_T_par);


//Declaring the prototype of the function returing a structure containing the data
//This function dsge_read_in_data is definied in DSGE_io.c
extern dsge_data_obj *dsge_read_in_data(char *inputfilename, int data_T);

	int main(int argc, char* argv[])
{
	int data_T = 81; // the length of the time-series in the data structure data_X
	int MH_type = 1; // RW is 0; Tailoring is 1; ARMH is 2; Randomizing over Tailoring and ARMH is 3
	int MH_T = 10000; // The length of MH iterations
	int i; //Some dummy indices

//Data filename located in the current directory
	char inputfilename[15] = "DSGE_data8.txt"; 
//Declaring the data structure member data_X
	dsge_data_obj *data_X;

gsl_vector * gradient = gsl_vector_alloc(17);

DSGE_like_params *param = (DSGE_like_params *) malloc( (unsigned) sizeof(DSGE_like_params));
gsl_vector * v_THETA_MLE = gsl_vector_alloc(17);//declaring a gsl vector of parameter MLE
gsl_vector *v_x = gsl_vector_alloc(17); //used to store a 17-dim point at which gradient is calculated

//Declaring the parameter Array = pointer to a parameter structure THETA of structure type dsge_param
dsge_param *THETA = (dsge_param*) malloc( (unsigned) (MH_T+3)*sizeof(dsge_param));

//Read in data
    data_X = dsge_read_in_data(inputfilename, data_T);
/*	printf("data_X =\n");
	for (i= 0;i<data_T;i++)
	{
		printf("%d   %.8g   %.8g   %.8g   %.8g \n",
		data_X[i].data_date, data_X[i].data_r , data_X[i].data_pi, data_X[i].data_gdp, data_X[i].data_delta);
	}*/

/*Initializing the THETA vector:*/
THETA[0].theta_tau	=	0.6543727	;
THETA[0].theta_beta	=	0.984504	;
THETA[0].theta_k	=	0.4078261	;
THETA[0].theta_psi1	=	2.3	;
THETA[0].theta_psi2	=	0.18606033	;
THETA[0].theta_rho_pi	=	0.6	;
THETA[0].theta_rho_r	=	0.9447543	;
THETA[0].theta_rho_g	=	0.77017574	;
THETA[0].theta_rho_z	=	0.69637463	;
THETA[0].theta_D     	=	0.26	;
THETA[0].theta_pi_star	=	4.1480042	;
THETA[0].theta_r_star	=	1.7176856	;
THETA[0].theta_sigma_r	=	0.078198755	;
THETA[0].theta_sigma_z	=	1.0949033	;
THETA[0].theta_sigma_g	=	0.13434636	;
THETA[0].theta_sigma_pi	=	0.23088188	;
THETA[0].theta_rho_gz	=	0.15864482	;




//Initializing vector of parameter MLE estimates (these values will be used to initialize BFGS)
for (i=0;i<17;i++)
	{
	gsl_vector_set(v_THETA_MLE,i,*(&THETA[0].theta_tau+i));
	}

//set parameter structure used in BFGS and MH (or just to evaluate the Gradient at some 17-dim point)
	(*param).data_T = data_T;
	(*param).data_X = data_X;
	(*param).partial = 17;
	(*param).dim = 17;
	(*param).v_x = v_x;


//******Maximization related code. MLE can be used as mean and Hessian evaluated at MLE as covariance matrix
//in M-H algorithm, or just Hessian to get Covariance for the proposal for R-W M-H as Shorfiede did.

printf("lglike(THETA, data_X, 0, data_T) = %.8g \n", lglike_fn(THETA, data_X, 0, data_T));
//SimAnneal(&THETA[0], v_THETA_MLE, data_X, data_T);//See comment below gsl_siman_solve in DSGE_SimAn.c file

BFGS(param, v_THETA_MLE);
print_scr_gsl_vec (v_THETA_MLE,17);

//Gradient calculation
DSGE_like_df (v_THETA_MLE, param, gradient);
printf("Gradient using my Looping through Mark's NR adaptation \n");
print_scr_gsl_vec (gradient,17);

//******************************Maximization-related code ended********************************
/*
gsl_vector_set(v_THETA_MLE,0,0.25112);
gsl_vector_set(v_THETA_MLE,1,0.989889);
gsl_vector_set(v_THETA_MLE,2,0.411561);
gsl_vector_set(v_THETA_MLE,3,2.18994);
gsl_vector_set(v_THETA_MLE,4,0.273756);
gsl_vector_set(v_THETA_MLE,5,0.776966);
gsl_vector_set(v_THETA_MLE,6, 0.881905);
gsl_vector_set(v_THETA_MLE,7,0.857269);
gsl_vector_set(v_THETA_MLE,8,0.715119);
gsl_vector_set(v_THETA_MLE,9,0.102558);
gsl_vector_set(v_THETA_MLE,10,2.58333);
gsl_vector_set(v_THETA_MLE,11,1.7588);
gsl_vector_set(v_THETA_MLE,12,0.0817405);
gsl_vector_set(v_THETA_MLE,13,0.682104);
gsl_vector_set(v_THETA_MLE,14,0.106683);
gsl_vector_set(v_THETA_MLE,15,0.224932);
gsl_vector_set(v_THETA_MLE,16,0.833905);
*/
MH_Tailoring(param, THETA, v_THETA_MLE, MH_T, MH_type);

// the following function will display all memory that has been allocated by not freed
//	_CrtDumpMemoryLeaks();

printf("\n WAKE UP! PRESS SOMETHING! IT'S DONE!");
getchar();



 return 0;
}


