#include "stdafx.h"


// Function for printing a GSL vector to screen
void print_scr_gsl_vec (gsl_vector * GSL_vec, int c);

// Functions and structure that make up the "gsl_multimin_function_fdf" type DSGE_like_FDF function
double DSGE_like_f (const gsl_vector *v_THETA, void *p);
void DSGE_like_df (const gsl_vector *v_THETA, void *p, gsl_vector *gradient);
void DSGE_like_fdf (const gsl_vector *v_THETA, void *p, double *f, gsl_vector *gradient);

void BFGS(DSGE_like_params *param, gsl_vector *v_THETA_MLE_par);

void BFGS(DSGE_like_params *param, gsl_vector *v_THETA_MLE_par)
{
//Declaring Variables used in BFGS algorithm
  int i;
  size_t iter = 0;
  int status;
  const gsl_multimin_fdfminimizer_type *T;
  gsl_multimin_fdfminimizer *s;

  	clock_t start1=0,stop1=0; //Speed timing variables
	double Time=0;

//Defining a function type, which contains in itself a function of high-dim argument and gradient
gsl_multimin_function_fdf DSGE_like_FDF;

//******************************Set up for BFGS routine in GSL*********************************////
  DSGE_like_FDF.f = &DSGE_like_f;
  DSGE_like_FDF.df = &DSGE_like_df;
  DSGE_like_FDF.fdf = &DSGE_like_fdf;
  DSGE_like_FDF.n = 17;
  DSGE_like_FDF.params = (void *) param;

  T = gsl_multimin_fdfminimizer_vector_bfgs;
  s = gsl_multimin_fdfminimizer_alloc (T, 17);

  gsl_multimin_fdfminimizer_set (s, &DSGE_like_FDF, v_THETA_MLE_par, 0.01, 1e-4);

  	start1=clock();
  do
    {
      iter++;
      status = gsl_multimin_fdfminimizer_iterate (s);

      if (status)
        break;

      status = gsl_multimin_test_gradient (s->gradient, 1e-3);

      if (status == GSL_SUCCESS)
        printf ("Minimum found at:\n");
	
	  printf ("%5d ", iter);
		for (i=0;i<17;i++)     
	       printf ("%.5f ", gsl_vector_get (s->x, i));
        printf ("%10.5f\n", s->f);

    }
  while (status == GSL_CONTINUE && iter < 3);

for (i=0;i<17;i++)     
	 gsl_vector_set(v_THETA_MLE_par, i, gsl_vector_get (s->x, i));//Saving MLE estimate in v_THETA_MLE_par

  gsl_multimin_fdfminimizer_free (s);
  	stop1=clock();
	Time=(1.0*(stop1 - start1)) / (1.0*CLOCKS_PER_SEC);
	printf(" BFGS. \n Elapsed time = %g sec \n",Time);

//Outputting the results to *.txt files under the project folder
	{
unsigned int i_f;
FILE * f_f;
 
f_f = fopen("MLE_BFGS.txt", "w");
for (i_f=0;i_f<(*v_THETA_MLE_par).size;i_f++)
      fprintf (f_f, "%.8g \n", gsl_vector_get(v_THETA_MLE_par,i_f) );
fclose (f_f);
	}   
      

  }