#include <stdio.h> 
#include <memory.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>

//#include "mkl.h"
#include <gsl/gsl_cblas.h> //Note that MKL and GSL libraries interfere here.
#include <gsl/gsl_math.h>
#include <gsl/gsl_vector.h>
#include <gsl/gsl_linalg.h>
#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>

//Defining the PARAMETER structure type
typedef struct
{
	double theta_tau;
	double theta_beta;
	double theta_k;
	double theta_psi1;
	double theta_psi2;

	double theta_rho_pi;
	double theta_rho_r;
	double theta_rho_g;
	double theta_rho_z;

	double theta_D;

	double theta_pi_star;
	double theta_r_star;

	double theta_sigma_r;
	double theta_sigma_z;
	double theta_sigma_g;
	double theta_sigma_pi;
	double theta_rho_gz;
} dsge_param;

// Defining the structure type that will be used to construct data
typedef struct
{
    int data_date;           /* quarter number */
    double data_r;           /* target interest rate */
    double data_pi;          /* inflation rate  */
    double data_gdp;         /* HP-filtered GDP */
    double data_delta;       /* discreteness adjustment */

} dsge_data_obj;

// Matrix form conversion function from MKL to GSL of size (GSL_row x GSL_col)
void MKLtoGSL (double MKL_mat[], gsl_matrix * GSL_mat, int GSL_row, int GSL_col);

// Matrix form conversion function from GSL of size (GSL_row x GSL_col) to MKL
void GSLtoMKL (gsl_matrix * GSL_mat, double MKL_mat[], int GSL_row, int GSL_col);

// Function for printing GSL matrix to screen
void print_scr_gsl_mat (gsl_matrix * GSL_mat, int r, int c);

// Function for printing a GSL vector to screen
void print_scr_gsl_vec (gsl_vector * GSL_vec, int c);

// Function for printing MKL matrix to screen
void print_scr_mkl_mat (double MKL_mat[], int r, int c);

// Function for setting the first n elements of an array a equal to the first n elements of array b
void eqarrays(double a[], double b[], int n);

extern double lglike_fn(dsge_param *THETA_par, dsge_data_obj *data_X_par, int MH_i_par, int data_T_par);

void MH_RandWalk(dsge_data_obj *data_X, dsge_param *THETA, gsl_vector *v_THETA_MLE, int MH_T, int data_T);

void MH_RandWalk(dsge_data_obj *data_X, dsge_param *THETA, gsl_vector *v_THETA_MLE, int MH_T, int data_T)
{
	int i,j;
	int MH_i; //MH iteration loop index variable
	int MT_i; // Multivariate T-distribution index
	double MH_alpha; //M-H acceptance probability
	double MH_u; //used in M-H uniform draw
	double accept_rate=0.0; //Acceptnce rate in the M-H algorithm
	double accept_indicat; //Indicator of Acceptance variable
	double burnin = 0.1; //Burn-in rate in M-H algorithm (proportion of discarded draws in the beginning)

//M-H proposal density variables:
	double v_MT[17]; //Vector drawn from multivariate T mean 0, Covariance I, d.f. MT_mu
	double v_MH_mean[17]; //Vector of proposal density's mean
	double MT_mu = 8.0; // Multivariate T degrees of freedom
	gsl_matrix * m_Cov = gsl_matrix_alloc(17,17); //Covariance Matrix for proposal density
	double MKL_Cov[17*17]; //The same matrix only in MKL form
	double THETA_MH_i1[17]; //Draw from the proposal density

	gsl_rng * r = gsl_rng_alloc (gsl_rng_taus); //Setting GSL random number generator

  	clock_t start1=0,stop1=0; //Speed timing variables
	double Time=0;

//Setting the Proposal Density Mean vector
	for (i=0;i<17;i++)
	{
//Setting the Proposal Density Mean vector
//v_MH_mean[i] = *(&THETA[0].theta_tau+i);
//v_MH_mean[i] = gsl_vector_get(v_THETA_MLE,i);
//Initializing the SUM of all draws for each variable used to calculate the mean of all the draws
*(&THETA[MH_T+1].theta_tau+i) = *(&THETA[0].theta_tau+i);
	}

/*Initializing the Covariance Matrix for the proposal density*/
gsl_matrix_set_identity(m_Cov);
gsl_matrix_set(m_Cov,0,0,0.15*0.15); gsl_matrix_set(m_Cov,1,1,0.01*0.01);
gsl_matrix_set(m_Cov,2,2,0.2*0.2); gsl_matrix_set(m_Cov,3,3,0.5*0.5);
gsl_matrix_set(m_Cov,4,4,0.15*0.15); gsl_matrix_set(m_Cov,5,5,0.12*0.12);
gsl_matrix_set(m_Cov,6,6,0.03*0.03); gsl_matrix_set(m_Cov,7,7,0.1*0.1);
gsl_matrix_set(m_Cov,8,8,0.1*0.1); gsl_matrix_set(m_Cov,9,9,0.3*0.3);
gsl_matrix_set(m_Cov,10,10,2.0*2.0); gsl_matrix_set(m_Cov,11,11,1.0*1.0);
gsl_matrix_set(m_Cov,12,12,0.02*0.02); gsl_matrix_set(m_Cov,13,13,0.1*0.1);
gsl_matrix_set(m_Cov,14,14,0.02*0.02); gsl_matrix_set(m_Cov,15,15,0.02*0.02);
gsl_matrix_set(m_Cov,16,16,0.3*0.3);

//************Preparing Hessian matrix for M-H algorithm****************************************


gsl_linalg_cholesky_decomp(m_Cov);//this stupid routine on output puts in the diagonal and lower triangular part of the input
								  //matrix m_Cov the matrix L and in the upper triangular part L^T, so:
for (i=0;i<16;i++)
{
	for(j=i+1;j<17;j++)
	gsl_matrix_set(m_Cov,i,j,0.0);
}
GSLtoMKL (m_Cov, MKL_Cov, 17, 17);


//*********************The main MH iteration loop will begin here.********************************************
	start1=clock();
for (MH_i= 0;MH_i<MH_T;MH_i++)
{

//Setting the Proposal Density Mean vector
	for (i=0;i<17;i++)
		v_MH_mean[i] = *(&THETA[MH_i].theta_tau+i);

//Draw from "standard" T 17-dimensional distribution:
	for (MT_i= 0;MT_i<17;MT_i++)
	{
	//using pathetic A-R method:
	do
	v_MT[MT_i] = gsl_ran_tdist(r, MT_mu);
	while(v_MT[MT_i]<-2.0);
	}

//THETA_[MH_i+1] = dgemm(m_Cov*v_MT + v_MH_mean)
	eqarrays(THETA_MH_i1, v_MH_mean, 17);
	cblas_dgemm (CblasColMajor, CblasNoTrans, CblasNoTrans, 17, 1, 17, 1.0, MKL_Cov, 17, v_MT, 17, 1.0, THETA_MH_i1, 17);
for (i=0;i<17;i++)
*(&THETA[MH_i+1].theta_tau+i) = THETA_MH_i1[i];
accept_indicat = 1.0;

MH_alpha = exp(lglike_fn(THETA, data_X, MH_i+1, data_T) - lglike_fn(THETA, data_X, MH_i, data_T));
if(!(MH_alpha>1.0))
	{
	MH_u = gsl_rng_uniform(r);
	if(MH_u > MH_alpha)
		{
		//THETA_[MH_i+1]=THETA_[MH_i]
		for (i=0;i<17;i++)
		*(&THETA[MH_i+1].theta_tau+i) = *(&THETA[MH_i].theta_tau+i);
		accept_indicat = 0.0;
		}
	}
/*
//Output to screen:
printf("%d   ",MH_i+1);
for (i=0;i<17;i++)
printf("%.8g  ",*(&THETA[MH_i+1].theta_tau+i));
printf("\n");
*/
//Adding up the draws and adding acceptance rate to get the averages
if ((double)MH_i>=(double)MH_T*burnin)
	{
	for (i=0;i<17;i++)
	*(&THETA[MH_T+1].theta_tau+i) =  *(&THETA[MH_T+1].theta_tau+i) + *(&THETA[MH_i+1].theta_tau+i);
	accept_rate = accept_rate + accept_indicat;
	}


} //The end of the main MH iteration loop ********************************************************************

//Output the mean to screen:
printf("M-H Mean ");
for (i=0;i<17;i++)
printf("%.8g ",*(&THETA[MH_T+1].theta_tau+i)/(MH_T*(1-burnin)));
printf("\n The acceptance rate is %.8g \n",accept_rate/(MH_T*(1-burnin)));

	stop1=clock();
	Time=(1.0*(stop1 - start1)) / (1.0*CLOCKS_PER_SEC);
	printf(" M-H algorithm Elapsed time = %g sec \n",Time);
  }