#include "stdafx.h"

/*Note that blas_dgemm declared in gsl_blas.h doesn't allow COLUMN-major matrix storage option*/
#include <gsl/gsl_cblas.h> //Note that MKL and GSL libraries interfere here.
#include <gsl/gsl_math.h>

#include"dfridders.h"


// Matrix form conversion function from MKL to GSL of size (GSL_row x GSL_col)
void MKLtoGSL (double MKL_mat[], gsl_matrix * GSL_mat, int GSL_row, int GSL_col);

// Matrix form conversion function from GSL of size (GSL_row x GSL_col) to MKL
void GSLtoMKL (gsl_matrix * GSL_mat, double MKL_mat[], int GSL_row, int GSL_col);

// Function for printing GSL matrix to screen
void print_scr_gsl_mat (gsl_matrix * GSL_mat, int r, int c);

// Function for printing a GSL vector to screen
void print_scr_gsl_vec (gsl_vector * GSL_vec, int c);

// Function for printing MKL matrix to screen
void print_scr_mkl_mat (double MKL_mat[], int r, int c);

// Function for setting the first n elements of an array a equal to the first n elements of array b
void eqarrays(double a[], double b[], int n);

extern double lglike_fn(dsge_param *THETA_par, dsge_data_obj *data_X_par, int MH_i_par, int data_T_par);

//Note that this function produces a NEGATIVE of likelihood. So, the resulting Hessian is a negative Hessian!
extern double DSGE_like_f (const gsl_vector *v_THETA, void *p);
int hessian( const gsl_vector* x, double (*f)( const gsl_vector*, void* ), void* params, double h, gsl_matrix* H );

//Modified Cholesky decomp using a Matlab compiled .dll
void matlabtoc_modchol(double *MKL_in, int dim, double *MKL_outres, double *MKL_outerr, int *i_outstate);

//Draw from N-dimensional Student-t
void MT_sampler(double * v_MH_mean, double * MKL_Chol_Cov, double scale, double MT_mu, int N, gsl_rng * _gsl_r, double *mult_t);

// Calculates Kernel of Multivariate Student-T pdf
double MT_kernel(double * v_MH_mean, double * MKL_T_InvCov,
				 double scale, double MT_mu, int N, double *THETA_MH_i);

//AutoCorrelation Function for M-H diagnostics
extern double *autocorr(double *data,int n,int l);

// Function that fills in the KF implied i_par(th) state values for all times given MH_i_par draw
void state_average(dsge_param *THETA_par, dsge_data_obj *data_X_par, int MH_i_par, int data_T_par,
				   gsl_matrix * GSL_states1, int i1_par, gsl_matrix * GSL_states2, int i2_par, int i_burnin);
extern double h; //Reference to global derivative step size h defined on the next line

void MH_Tailoring(DSGE_like_params * param, dsge_param *THETA, gsl_vector *v_THETA_MLE, int MH_T, unsigned int Tailor);

void MH_Tailoring(DSGE_like_params * param, dsge_param *THETA, gsl_vector *v_THETA_MLE, int MH_T, unsigned int Tailor)

{
	int i,j;//various indecies
	int MH_i; //MH iteration loop index variable
	double MH_alpha; //M-H acceptance probability
	double AR_alpha; //A-R acceptance probability
	double MH_u; //used in M-H uniform draw
	double accept_rate=0.0; //Acceptnce rate in the M-H algorithm Overall
	double accept_indicat=1.0; //Indicator of Acceptance variable
	double AR_counter = 0.0;//Counter of A-R iterations
	unsigned int Tailortemp = Tailor;//Used for randomizing over algorithms
	double ARMH_counter=0.0; // Counts the total number of ARMH draws out of ARMH+Tailoring draws
	double burnin = 0.1; //Burn-in rate in M-H algorithm (proportion of discarded draws in the beginning)
	int lags = 200; //Maximum number of lags for ACF series
	gsl_matrix * ACF = gsl_matrix_alloc(lags+1,17);//Pointer to AutoCorr Function Structure
	int i_burnin = (int) (MH_T*burnin); //Integer number of burnin draws
	double *MH_par = new double [MH_T - i_burnin]; //Uses in ACF calculation 
	double v_MH_mean[17], v_MH_meantemp[17]; //Vector of proposal density's mean
	gsl_matrix * m_Cov = gsl_matrix_alloc(17,17); //Covariance Matrix for proposal density
	double MKL_T_InvCov[17*17]; //INVERSE of m_Cov only in MKL form
	double MKL_m_Cov[17*17];
	double mchol_err[17*17];
	int *MKL_mchol_state = new int [1];
	double MKL_Chol_Cov[17*17]; //"transpose(sqrt(m_Cov))" by Cholesky decomposition
	double THETA_MH_i0[17]; //Array needed to feed to MT_kernel to calculate qold
	double THETA_MH_i1[17]; //Draw from the proposal density

/*********Tuning variables****************/
	double RW_scale=1.0, RW_scaletemp;//scale for the "square root" of the covariance matrix
	double MT_mu = 15.0, MT_mutemp; // Multivariate T degrees of freedom
	double AR_c = 1.5; /*tuning constant in the Accept-Reject step*/
	double MH_type_prob = 0.25; /*Probability of chosing Tailoring*/
/*****************************************/

	double temp1, temp2, temp3;/*used as temporary holders*/
	double pold, qold, pnew, qnew; /*used in MH step of A-R Tailoring to determine aplha*/
	int C1, C2;

  	clock_t start1=0,stop1=0,stop2=0,stop3=0; //Speed timing variables
	double Time=0;

	dsge_data_obj *data_X;
	int data_T;
	data_X = (*param).data_X;
	data_T = (*param).data_T;

	int i1,i2;
	gsl_matrix * GSL_states1 = gsl_matrix_alloc ((MH_T-i_burnin), data_T);
	gsl_matrix * GSL_states2 = gsl_matrix_alloc ((MH_T-i_burnin), data_T);

	gsl_matrix * negHessian = gsl_matrix_alloc(17,17);
	gsl_permutation * permut = gsl_permutation_alloc (17);
	int signum;

//C++ Hessian/Gradient Code - related variables
	gsl_vector * g,* error;
  struct Dfridders* Df = get_dfridders( 0 );
  g = gsl_vector_alloc( 17 );
  error = gsl_vector_alloc( 17 );

	for (i=0;i<17;i++)
	{
/*Setting the Proposal Density Mean vector
 v_MH_mean[i] = *(&THETA[0].theta_tau+i);*/
v_MH_mean[i] = gsl_vector_get(v_THETA_MLE,i);
/*Setting to zero the sum used to calculate the mean of MH draws*/
*(&THETA[MH_T+1].theta_tau+i) = 0.0;
/*setting the first elements of THETA equal to MLE
  (it will be overwritten at the first iteration anyway,
  but I need it only before that to calculate AR_c*/
*(&THETA[1].theta_tau+i) = gsl_vector_get(v_THETA_MLE,i);
	}

//NEGATIVE Hessian of likelihood:
hessian( v_THETA_MLE, &DSGE_like_f, param, h, negHessian );
//both hessian( v_THETA_MLE, &DSGE_like_f,... AND hessian( v_THETA_MLE, DSGE_like_f,...
//will work and produce the same result (see email from John Lamb for explanation)
															 
printf("Negative Hessian: \n");
print_scr_gsl_mat (negHessian, 17, 17);

dfridders( Df, g, error, &DSGE_like_f, v_THETA_MLE, param, h );
printf("\n Gradient Using C++ code from John Lamb: \n");
print_scr_gsl_vec (g,17);
printf("Gradient Errors \n");
print_scr_gsl_vec (error,17);
  gsl_vector_free( error );
  gsl_vector_free( g );

/*retaining the INVERSE of covariance Matrix of Tailored Proposal Density
  to be used later in the Mult T kernel function*/
GSLtoMKL(negHessian, MKL_T_InvCov, 17, 17);

/*Calculating the AR_c constant as in C-G-W (1998)*/
temp1 = lglike_fn(THETA, data_X, 1, data_T);
temp2 = log(MT_kernel(v_MH_mean, MKL_T_InvCov, pow(RW_scale,-2.0), MT_mu, 17, v_MH_mean));
AR_c = exp(log(AR_c) +temp1 - temp2);

/*Inverting the negative Hessian to get the covariance matrix for proposal*/
gsl_linalg_LU_decomp (negHessian, permut, &signum);
gsl_linalg_LU_invert (negHessian, permut, m_Cov);
printf("Covariance Matrix of Tailored Proposal Density: \n");
print_scr_gsl_mat (m_Cov, 17, 17);
printf("Mean of Tailored Proposal Density: \n");
print_scr_mkl_mat(v_MH_mean, 17, 1);

gsl_matrix_free (negHessian);
gsl_permutation_free (permut);
//************Preparing Hessian matrix for M-H algorithm****************************************
GSLtoMKL(m_Cov, MKL_m_Cov, 17, 17);
matlabtoc_modchol(MKL_m_Cov, 17, MKL_Chol_Cov, mchol_err, MKL_mchol_state);
printf("Modified Cholesky lower triangular matrix L \n");
print_scr_mkl_mat(MKL_Chol_Cov, 17, 17);
printf("Modified Cholesky error matrix E \n");
print_scr_mkl_mat(mchol_err, 17, 17);
printf("Modified Cholesky state = %d \n", *MKL_mchol_state);
delete [] MKL_mchol_state;
//gsl_linalg_cholesky_decomp(m_Cov);//this stupid routine on output puts in the diagonal and lower triangular part of the input
								  //matrix m_Cov the matrix L and in the upper triangular part L^T, so:
//for (i=0;i<16;i++)
//{
//	for(j=i+1;j<17;j++)
//	gsl_matrix_set(m_Cov,i,j,0.0);
//}
//GSLtoMKL (m_Cov, MKL_Chol_Cov, 17, 17);
gsl_matrix_free (m_Cov);

/*********************The main MH iteration loop will begin here.********************************/
	start1=clock();
for (MH_i= 0;MH_i<MH_T;MH_i++)
{
	/*CGW JE (1998) mixed over RW and Tailoring. On p. 38 it says:
	MARKOV PROPERTY OF THE SIMULATION IS PRESERVED IF THE RESPECTIVE
	DENSITIES ARE SELECTED AT FIXED PRESPECIFIED INTERVALS */

	/*Randomize over Tailoring and ARMH in the following way:*/
	if (Tailortemp == 3)
		{
			MH_u = gsl_rng_uniform(_gsl_r);
			if(MH_u > MH_type_prob)
			Tailor = 2;
			else
			Tailor = 0;
		}

/*Counting the percentage of ARMH draws of the total Tailoring + ARMH draws*/
		if (Tailor == 2)
			ARMH_counter = ARMH_counter + 1.0;



/*AR-Tailored MH implementation*/
	if (Tailor == 2)
		{
		for (i=0;i<17;i++)/*Need it to evaluate qold in A-R MH step*/
			THETA_MH_i0[i] = *(&THETA[MH_i].theta_tau+i);
		do
			{
			/*Drawing from N-dim Multivariate Student-t proposal*/
			MT_sampler(v_MH_mean, MKL_Chol_Cov, RW_scale, 
					   MT_mu, 17, _gsl_r, THETA_MH_i1);
			//print_scr_mkl_mat(THETA_MH_i1, 17, 1);
			for (i=0;i<17;i++)
				*(&THETA[MH_i+1].theta_tau+i) = THETA_MH_i1[i];
	
			accept_indicat = 1.0;
			AR_counter = AR_counter +1.0;

			/*Check to make sure that do-while loop doesn't run for ever:*/
			if (AR_counter > 10.0* (double) MH_T)
				{
				printf("AR_counter = %.8g \n A-R do-while loop is too long \n", AR_counter);
				getchar();
				exit(1);
				}

			AR_alpha = lglike_fn(THETA, data_X, MH_i+1, data_T);
			temp3 = log(AR_c*MT_kernel(v_MH_mean, MKL_T_InvCov, pow(RW_scale,-2.0), MT_mu, 17, THETA_MH_i1));
			AR_alpha = exp(AR_alpha - temp3);

			if(!(AR_alpha>1.0))
				{
				MH_u = gsl_rng_uniform(_gsl_r);
				if(MH_u > AR_alpha)
					accept_indicat = 0.0;
				}
			}
		while(accept_indicat == 0.0);
		/*Adding up the acceptance rate for A-R step is not needed b/c by construction
		  Total acceptance = MH_T - eventually every i_MH gets accepted due to do-while loop*/
		
		pold = lglike_fn(THETA, data_X, MH_i, data_T);
		pnew = lglike_fn(THETA, data_X, MH_i+1, data_T);
		qold = log(MT_kernel(v_MH_mean, MKL_T_InvCov, pow(RW_scale,-2.0), MT_mu, 17, THETA_MH_i0));
		qnew = log(MT_kernel(v_MH_mean, MKL_T_InvCov, pow(RW_scale,-2.0), MT_mu, 17, THETA_MH_i1));
		
		C1 = 0; C2 = 0;
		if (pold < (log(AR_c) + qold))
			C1 = 1;
		if (pnew < (log(AR_c) + qnew))
			C2 = 1;

		if (C1 == 1)
			MH_alpha = 1.0;
		if ((C1 == 0) && (C2 == 1))
			MH_alpha = exp(log(AR_c) + qold - pold );
		if ((C1 == 0) && (C2 == 0))
			MH_alpha = exp(pnew - pold + qold - qnew );
		
		if(!(MH_alpha>0.999999))
			{
			MH_u = gsl_rng_uniform(_gsl_r);
			if(MH_u > MH_alpha)
				{
				//THETA_[MH_i+1]=THETA_[MH_i]
				for (i=0;i<17;i++)
					*(&THETA[MH_i+1].theta_tau+i) = *(&THETA[MH_i].theta_tau+i);
				accept_indicat = 0.0;
				}
			}
		}

/*Random Walk or Tailored MH implementation*/
if (Tailor == 0 || Tailor == 1)
	{

/*In order to turn Tailoring into RW M-H, one simply needs to change the mean of the proposal density,
  and use Normal instead of T. This is what is done below if Tailor == 0*/
	if (Tailor == 0)
	{
	for (i=0;i<17;i++)
		{
		v_MH_meantemp[i] = v_MH_mean[i];
		v_MH_mean[i] = *(&THETA[MH_i].theta_tau+i);//Setting the Proposal Density Mean vector
		}
	MT_mutemp = MT_mu; 
	MT_mu = 50.0;//Turning Student-T into Normal
	RW_scaletemp = RW_scale;
	RW_scale = 0.5;//scaling the "square root" of the Covariance matrix
	}

	/*Drawing from N-dim Multivariate Student-t proposal*/
	MT_sampler(v_MH_mean, MKL_Chol_Cov, RW_scale, 
			   MT_mu, 17, _gsl_r, THETA_MH_i1);
	//print_scr_mkl_mat(THETA_MH_i1, 17, 1);
	for (i=0;i<17;i++)
		*(&THETA[MH_i+1].theta_tau+i) = THETA_MH_i1[i];

	accept_indicat = 1.0;
	pnew = lglike_fn(THETA, data_X, MH_i+1, data_T);
	pold = lglike_fn(THETA, data_X, MH_i, data_T);
	qold = 0.0; qnew = 0.0;
	if (Tailor == 1)
		{
		for (i=0;i<17;i++)/*Need it to evaluate qold in below*/
			THETA_MH_i0[i] = *(&THETA[MH_i].theta_tau+i);
		qold = log(MT_kernel(v_MH_mean, MKL_T_InvCov, pow(RW_scale,-2.0), MT_mu, 17, THETA_MH_i0));
		qnew = log(MT_kernel(v_MH_mean, MKL_T_InvCov, pow(RW_scale,-2.0), MT_mu, 17, THETA_MH_i1));
		}
	MH_alpha = exp(pnew - pold + qold - qnew );
	if(!(MH_alpha>1.0))
		{
		MH_u = gsl_rng_uniform(_gsl_r);
		if(MH_u > MH_alpha)
			{
			//THETA_[MH_i+1]=THETA_[MH_i]
			for (i=0;i<17;i++)
			*(&THETA[MH_i+1].theta_tau+i) = *(&THETA[MH_i].theta_tau+i);
			accept_indicat = 0.0;
			}
		}

	if ((Tailortemp == 3) && (Tailor == 0))
		{
/*Restoring RW-specific parameters to the general values*/
	for (i=0;i<17;i++)
		v_MH_mean[i] = v_MH_meantemp[i];
	MT_mu = MT_mutemp; 
	RW_scale = RW_scaletemp;
		}

	}

/*
//Output to screen:
printf("%d   ",MH_i+1);
for (i=0;i<17;i++)
printf("%.8g  ",*(&THETA[MH_i+1].theta_tau+i));
printf("\n");
*/
//Adding up the draws and adding acceptance rate to get the averages
if ((MH_i>i_burnin) || (MH_i == i_burnin))
	{
	//printf("THETA_tau[%d] = %.8g \n",MH_T+1,*(&THETA[MH_T+1].theta_tau));
	for (i=0;i<17;i++)
	*(&THETA[MH_T+1].theta_tau+i) =  *(&THETA[MH_T+1].theta_tau+i) + *(&THETA[MH_i+1].theta_tau+i);
	accept_rate = accept_rate + accept_indicat;
	}


} //The end of the main MH iteration loop ********************************************************************



/*Calculating the standard deviation of MH draws for all parameters
  AND turning the sum of the parameter draws contained in MH_T+1 to the mean*/
for (i=0;i<17;i++)
	{
/*Dividing the Sum used to compute mean over the number of draws used*/
	*(&THETA[MH_T+1].theta_tau+i) = *(&THETA[MH_T+1].theta_tau+i)/(MH_T-i_burnin);
	*(&THETA[MH_T+2].theta_tau+i) = 0.0;/*Initializing the st.dev. sum*/
	for (MH_i= i_burnin+1;MH_i<MH_T+1;MH_i++)
		{/*sum of (mean-MH_i)^2 over all MH_i draws for each i(th) variable*/
		*(&THETA[MH_T+2].theta_tau+i) = *(&THETA[MH_T+2].theta_tau+i)+
		pow((*(&THETA[MH_T+1].theta_tau+i) - *(&THETA[MH_i].theta_tau+i)),2.0);
		}
/*Dividing the Sum used to compute st. dev. over the number of draws used-1 and taking sqrt*/
	*(&THETA[MH_T+2].theta_tau+i) = sqrt(*(&THETA[MH_T+2].theta_tau+i)/(MH_T-i_burnin-1));
	}



/*Output the mean and standard deviation to screen:*/
if (Tailortemp == 3)
printf("Tailor/ARMH randomization ");
else if (Tailor == 1)
printf("Tailor ");
else if (Tailor ==2)
printf("A-R Tailor ");
else
printf("R-W ");
printf("M-H \n  Mean      Standard Deviation \n");
for (i=0;i<17;i++)
	{
	printf("%.8g    %.8g \n",*(&THETA[MH_T+1].theta_tau+i),*(&THETA[MH_T+2].theta_tau+i));
	}
if ((Tailor == 2) || (Tailortemp == 3))
	printf("\n The A-R acceptance rate (including burn-in) is %.8g ",ARMH_counter/AR_counter);

printf("\n The M-H acceptance rate (excluding burn-in) is %.8g \n",accept_rate/(MH_T-i_burnin));
printf("\n The proportion of ARMH draws out of ARMH + Tailoring (including burn-in) is %.8g ",ARMH_counter/((double)MH_T));
printf("\n Number of M-H iterations (including burn-in) is %d \n",MH_T);
printf("\n Cholesky(Covar) scaling used is %.8g \n",RW_scale);

	stop1=clock();
	Time=(1.0*(stop1 - start1)) / (1.0*CLOCKS_PER_SEC);
	printf(" M-H algorithm Elapsed time = %g sec \n",Time);


/*Outputting the results to *.txt files under the project folder*/
/*Note that Mean and variance will be in the last 2 rows of "MH_RESULTS.txt" file*/
	{
	int i_f, MH_i_f;
	FILE * f_res;
	if (Tailortemp == 3)
	f_res = fopen("ARMH_Tailor_RESULTS.txt", "w");
	else if (Tailor == 1)
	f_res = fopen("MH_Tailor_RESULTS.txt", "w");
	else if (Tailor ==2)
	f_res = fopen("ARMH_RESULTS.txt", "w");
	else
 	f_res = fopen("MH_RW_RESULTS.txt", "w");

	for (MH_i_f= 0;MH_i_f<MH_T+3;MH_i_f++)
		{
		for (i_f=0;i_f<17;i_f++)
		fprintf (f_res, "%.8g   ", *(&THETA[MH_i_f].theta_tau+i_f) );
		fprintf (f_res, "\n");
		}
	fclose (f_res);
	}  



/*************Diagnostic of M-H algorithm**********/
/*AutoCorrelation Function*/
for (i=0;i<17;i++)
	{
	/*Creating the "data" array for autocorr function*/
	for (j=i_burnin;j<MH_T;j++)
		{
		MH_par[j- i_burnin] = *(&THETA[j].theta_tau+i);
		//printf("MH_par[%d] = %.8g \n",j,MH_par[j]);
		}
/*I have tried, but couldn't create a pointer to a column (or row) from ACF multidim array.
It is probably impossible, despite the fact that multidim array could be thought of as
"array of arrays" (see p.611)*/
	double *ACF_temp = new double [lags]; //pointer used to temporarily store ACF output
	ACF_temp = autocorr(MH_par, MH_T-i_burnin, lags);//to avoid multiple unnecessary calls to autocorr() function
	gsl_matrix_set(ACF, lags, i, -1.0);//b/c lag = 0 term should be excluded from inefficiency factor calculation below
	for (j=0;j<lags;j++)
		{
		gsl_matrix_set(ACF, j, i, ACF_temp[j]);
		gsl_matrix_set(ACF, lags, i, gsl_matrix_get(ACF, lags, i) + (1.0-((double)j)/(double(lags-1)))*ACF_temp[j]);
		}
	delete [] ACF_temp;
	gsl_matrix_set(ACF, lags, i, 1.0+2.0*gsl_matrix_get(ACF, lags, i));//inefficiency factor
	}

/*Output ACF to a file*/
	{
	int i_f, MH_i_f;
	FILE * f_acf;
	if (Tailortemp == 3)
	f_acf = fopen("ARMH_Tailor_ACF.txt", "w");
	else if (Tailor == 1)
	f_acf = fopen("MH_Tailor_ACF.txt", "w");
	else if (Tailor ==2)
	f_acf = fopen("ARMH_ACF.txt", "w");
	else
 	f_acf = fopen("MH_RW_ACF.txt", "w");

	for (MH_i_f= 0;MH_i_f<lags+1;MH_i_f++)
		{
		for (i_f=0;i_f<17;i_f++)
		fprintf (f_acf, "%.8g   ", gsl_matrix_get(ACF,MH_i_f,i_f) );
		fprintf (f_acf, "\n");
		}
	fclose (f_acf);
	}  
	gsl_matrix_free(ACF);

	stop2=clock();
	Time=(1.0*(stop2 - stop1)) / (1.0*CLOCKS_PER_SEC);
	printf(" ACF Elapsed time = %g sec \n",Time);

/*calculate average of the mean state variables conditional on MH draw at every time point*/
	i1 = 3;/*corresponds to the i(th) element in the state vector starting at i=0*/
	i2 = 5;
	for (MH_i= i_burnin+1;MH_i<MH_T+1;MH_i++)
		{	
		state_average(THETA, data_X, MH_i, data_T, GSL_states1, i1, GSL_states2, i2, i_burnin);
		}
	stop3=clock();
	Time=(1.0*(stop3 - stop2)) / (1.0*CLOCKS_PER_SEC);
	printf(" state_average Elapsed time = %g sec \n",Time);

/*Output state_average1 to a file*/
	{
	int i_f, MH_i_f;
	FILE * f_state;
	if (Tailortemp == 3)
	f_state = fopen("ARMH_Tailor_state1.txt", "w");
	else if (Tailor == 1)
	f_state = fopen("MH_Tailor_state1.txt", "w");
	else if (Tailor ==2)
	f_state = fopen("ARMH_state1.txt", "w");
	else
 	f_state = fopen("MH_RW_state1.txt", "w");

	for (MH_i_f= 0;MH_i_f<(MH_T-i_burnin);MH_i_f++)
		{
		for (i_f=0;i_f<data_T;i_f++)
		fprintf (f_state, "%.8g  ", gsl_matrix_get (GSL_states1, MH_i_f, i_f));
		fprintf (f_state, "\n");
		}
	fclose (f_state);
	}  

	gsl_matrix_free (GSL_states1);

/*Output state_average2 to a file*/
	{
	int i_f, MH_i_f;
	FILE * f2_state;
	if (Tailortemp == 3)
	f2_state = fopen("ARMH_Tailor_state2.txt", "w");
	else if (Tailor == 1)
	f2_state = fopen("MH_Tailor_state2.txt", "w");
	else if (Tailor ==2)
	f2_state = fopen("ARMH_state2.txt", "w");
	else
 	f2_state = fopen("MH_RW_state2.txt", "w");

	for (MH_i_f= 0;MH_i_f<(MH_T-i_burnin);MH_i_f++)
		{
		for (i_f=0;i_f<data_T;i_f++)
		fprintf (f2_state, "%.8g  ", gsl_matrix_get (GSL_states2, MH_i_f, i_f));
		fprintf (f2_state, "\n");
		}
	fclose (f2_state);
	}  

	gsl_matrix_free (GSL_states2);

  }