#include <stdio.h> 
#include <memory.h>
#include <math.h>
#include <stdlib.h>
#include <gsl/gsl_siman.h>
#include <time.h>
#include <gsl/gsl_math.h>
#include <gsl/gsl_vector.h>

/* set up parameters for this simulated annealing run */

/* how many points do we try before stepping */
#define N_TRIES 1000             

/* how many iterations for each T? */
#define ITERS_FIXED_T 50        

/* max step size in random walk */
#define STEP_SIZE 0.1            

/* Boltzmann constant */
#define K 1.0                   

/* initial temperature */
#define T_INITIAL 0.002         

/* damping factor for temperature */
#define MU_T 1.005              
#define T_MIN 2.0e-6

extern gsl_rng * _gsl_r; //Reference to global seed _gsl_r defined in DSGE.c

gsl_siman_params_t params 
  = {N_TRIES, ITERS_FIXED_T, STEP_SIZE,
     K, T_INITIAL, MU_T, T_MIN};

// Defining the structure type that will be used to construct data
typedef struct
{
    int data_date;           /* quarter number */
    double data_r;           /* target interest rate */
    double data_pi;          /* inflation rate  */
    double data_gdp;           /* HP-filtered GDP */
    double data_delta;       /* discreteness adjustment */
} dsge_data_obj;

//Defining the PARAMETER structure type
typedef struct
{
	double theta_tau;
	double theta_beta;
	double theta_k;
	double theta_psi1;
	double theta_psi2;

	double theta_rho_pi;
	double theta_rho_r;
	double theta_rho_g;
	double theta_rho_z;

	double theta_D;

	double theta_pi_star;
	double theta_r_star;

	double theta_sigma_r;
	double theta_sigma_z;
	double theta_sigma_g;
	double theta_sigma_pi;
	double theta_rho_gz;
} dsge_param;

//Creating a GLOBAL pointer to MLE estimate to use in the print function
gsl_vector * SIM_ANN_MLE = gsl_vector_alloc(17);
//Creating a GLOBAL pointer to data object and global int data length
	dsge_data_obj *gl_data_X;
	int gl_data_T;

extern double lglike_fn(dsge_param *THETA_par, dsge_data_obj *data_X_par, int MH_i_par, int data_T_par);
extern dsge_data_obj *dsge_read_in_data(char *inputfilename, int data_T);


/* The Energy function that we want to minimize (=-likelihood)*/
double E_like(void *xp)
{
dsge_param *THETA_x = (dsge_param*) malloc( (unsigned) sizeof(dsge_param));

   THETA_x[0].theta_tau = ((dsge_param *)xp)[0].theta_tau;
   THETA_x[0].theta_beta = ((dsge_param *)xp)[0].theta_beta;
   THETA_x[0].theta_k = ((dsge_param *)xp)[0].theta_k;
   THETA_x[0].theta_psi1 = ((dsge_param *)xp)[0].theta_psi1;
   THETA_x[0].theta_psi2 = ((dsge_param *)xp)[0].theta_psi2;
   THETA_x[0].theta_rho_pi = ((dsge_param *)xp)[0].theta_rho_pi;
   THETA_x[0].theta_rho_r = ((dsge_param *)xp)[0].theta_rho_r;
   THETA_x[0].theta_rho_g = ((dsge_param *)xp)[0].theta_rho_g;
   THETA_x[0].theta_rho_z = ((dsge_param *)xp)[0].theta_rho_z;
   THETA_x[0].theta_D = ((dsge_param *)xp)[0].theta_D;
   THETA_x[0].theta_pi_star = ((dsge_param *)xp)[0].theta_pi_star;
   THETA_x[0].theta_r_star = ((dsge_param *)xp)[0].theta_r_star;
   THETA_x[0].theta_sigma_r = ((dsge_param *)xp)[0].theta_sigma_r;
   THETA_x[0].theta_sigma_z = ((dsge_param *)xp)[0].theta_sigma_z;
   THETA_x[0].theta_sigma_g = ((dsge_param *)xp)[0].theta_sigma_g;
   THETA_x[0].theta_sigma_pi = ((dsge_param *)xp)[0].theta_sigma_pi;
   THETA_x[0].theta_rho_gz = ((dsge_param *)xp)[0].theta_rho_gz;
   
  return -lglike_fn(THETA_x, gl_data_X, 0, gl_data_T);
}

/*Metric function*/
double M_like(void *xp, void *yp)
{
	double distance_measure;
	dsge_param *THETA_x = (dsge_param*) malloc( (unsigned) sizeof(dsge_param));
	dsge_param *THETA_y = (dsge_param*) malloc( (unsigned) sizeof(dsge_param));
  (dsge_param *)xp;
  (dsge_param *)yp;
  
   THETA_x[0].theta_tau = ((dsge_param *)xp)[0].theta_tau;
   THETA_x[0].theta_beta = ((dsge_param *)xp)[0].theta_beta;
   THETA_x[0].theta_k = ((dsge_param *)xp)[0].theta_k;
   THETA_x[0].theta_psi1 = ((dsge_param *)xp)[0].theta_psi1;
   THETA_x[0].theta_psi2 = ((dsge_param *)xp)[0].theta_psi2;
   THETA_x[0].theta_rho_pi = ((dsge_param *)xp)[0].theta_rho_pi;
   THETA_x[0].theta_rho_r = ((dsge_param *)xp)[0].theta_rho_r;
   THETA_x[0].theta_rho_g = ((dsge_param *)xp)[0].theta_rho_g;
   THETA_x[0].theta_rho_z = ((dsge_param *)xp)[0].theta_rho_z;
   THETA_x[0].theta_D = ((dsge_param *)xp)[0].theta_D;
   THETA_x[0].theta_pi_star = ((dsge_param *)xp)[0].theta_pi_star;
   THETA_x[0].theta_r_star = ((dsge_param *)xp)[0].theta_r_star;
   THETA_x[0].theta_sigma_r = ((dsge_param *)xp)[0].theta_sigma_r;
   THETA_x[0].theta_sigma_z = ((dsge_param *)xp)[0].theta_sigma_z;
   THETA_x[0].theta_sigma_g = ((dsge_param *)xp)[0].theta_sigma_g;
   THETA_x[0].theta_sigma_pi = ((dsge_param *)xp)[0].theta_sigma_pi;
   THETA_x[0].theta_rho_gz = ((dsge_param *)xp)[0].theta_rho_gz;

   THETA_y[0].theta_tau = ((dsge_param *)yp)[0].theta_tau;
   THETA_y[0].theta_beta = ((dsge_param *)yp)[0].theta_beta;
   THETA_y[0].theta_k = ((dsge_param *)yp)[0].theta_k;
   THETA_y[0].theta_psi1 = ((dsge_param *)yp)[0].theta_psi1;
   THETA_y[0].theta_psi2 = ((dsge_param *)yp)[0].theta_psi2;
   THETA_y[0].theta_rho_pi = ((dsge_param *)yp)[0].theta_rho_pi;
   THETA_y[0].theta_rho_r = ((dsge_param *)yp)[0].theta_rho_r;
   THETA_y[0].theta_rho_g = ((dsge_param *)yp)[0].theta_rho_g;
   THETA_y[0].theta_rho_z = ((dsge_param *)yp)[0].theta_rho_z;
   THETA_y[0].theta_D = ((dsge_param *)yp)[0].theta_D;
   THETA_y[0].theta_pi_star = ((dsge_param *)yp)[0].theta_pi_star;
   THETA_y[0].theta_r_star = ((dsge_param *)yp)[0].theta_r_star;
   THETA_y[0].theta_sigma_r = ((dsge_param *)yp)[0].theta_sigma_r;
   THETA_y[0].theta_sigma_z = ((dsge_param *)yp)[0].theta_sigma_z;
   THETA_y[0].theta_sigma_g = ((dsge_param *)yp)[0].theta_sigma_g;
   THETA_y[0].theta_sigma_pi = ((dsge_param *)yp)[0].theta_sigma_pi;
   THETA_y[0].theta_rho_gz = ((dsge_param *)yp)[0].theta_rho_gz;
//I might want to revise the measure below, e.g. divide theta_pi_star, theta_r_star, and theta_psi1 lines by 10^2
distance_measure = sqrt((THETA_y[0].theta_tau - THETA_x[0].theta_tau)*(THETA_y[0].theta_tau - THETA_x[0].theta_tau)+
						(THETA_y[0].theta_beta - THETA_x[0].theta_beta)*(THETA_y[0].theta_beta - THETA_x[0].theta_beta)+
						(THETA_y[0].theta_k - THETA_x[0].theta_k)*(THETA_y[0].theta_k - THETA_x[0].theta_k)+
						(THETA_y[0].theta_psi1 - THETA_x[0].theta_psi1)*(THETA_y[0].theta_psi1 - THETA_x[0].theta_psi1)+
						(THETA_y[0].theta_psi2 - THETA_x[0].theta_psi2)*(THETA_y[0].theta_psi2 - THETA_x[0].theta_psi2)+
						(THETA_y[0].theta_rho_pi - THETA_x[0].theta_rho_pi)*(THETA_y[0].theta_rho_pi - THETA_x[0].theta_rho_pi)+
						(THETA_y[0].theta_rho_r - THETA_x[0].theta_rho_r)*(THETA_y[0].theta_rho_r - THETA_x[0].theta_rho_r)+
						(THETA_y[0].theta_rho_g - THETA_x[0].theta_rho_g)*(THETA_y[0].theta_rho_g - THETA_x[0].theta_rho_g)+
						(THETA_y[0].theta_rho_z - THETA_x[0].theta_rho_z)*(THETA_y[0].theta_rho_z - THETA_x[0].theta_rho_z)+
						(THETA_y[0].theta_D - THETA_x[0].theta_D)*(THETA_y[0].theta_D - THETA_x[0].theta_D)+
						(THETA_y[0].theta_pi_star - THETA_x[0].theta_pi_star)*(THETA_y[0].theta_pi_star - THETA_x[0].theta_pi_star)+
						(THETA_y[0].theta_r_star - THETA_x[0].theta_r_star)*(THETA_y[0].theta_r_star - THETA_x[0].theta_r_star)+
						(THETA_y[0].theta_sigma_r - THETA_x[0].theta_sigma_r)*(THETA_y[0].theta_sigma_r - THETA_x[0].theta_sigma_r)+
						(THETA_y[0].theta_sigma_z - THETA_x[0].theta_sigma_z)*(THETA_y[0].theta_sigma_z - THETA_x[0].theta_sigma_z)+
						(THETA_y[0].theta_sigma_g - THETA_x[0].theta_sigma_g)*(THETA_y[0].theta_sigma_g - THETA_x[0].theta_sigma_g)+
						(THETA_y[0].theta_sigma_pi - THETA_x[0].theta_sigma_pi)*(THETA_y[0].theta_sigma_pi - THETA_x[0].theta_sigma_pi)+
						(THETA_y[0].theta_rho_gz - THETA_x[0].theta_rho_gz)*(THETA_y[0].theta_rho_gz - THETA_x[0].theta_rho_gz));
  return distance_measure;
}

/*Step function*/
void S_like(const gsl_rng * _gsl_r, void *xp, double step_size)
{
	int i;
	double u[17];
dsge_param *THETA_new = (dsge_param*) malloc( (unsigned) sizeof(dsge_param));
dsge_param *THETA_x = (dsge_param*) malloc( (unsigned) sizeof(dsge_param));
  (dsge_param *)xp;
  
   THETA_x[0].theta_tau = ((dsge_param *)xp)[0].theta_tau;
   THETA_x[0].theta_beta = ((dsge_param *)xp)[0].theta_beta;
   THETA_x[0].theta_k = ((dsge_param *)xp)[0].theta_k;
   THETA_x[0].theta_psi1 = ((dsge_param *)xp)[0].theta_psi1;
   THETA_x[0].theta_psi2 = ((dsge_param *)xp)[0].theta_psi2;
   THETA_x[0].theta_rho_pi = ((dsge_param *)xp)[0].theta_rho_pi;
   THETA_x[0].theta_rho_r = ((dsge_param *)xp)[0].theta_rho_r;
   THETA_x[0].theta_rho_g = ((dsge_param *)xp)[0].theta_rho_g;
   THETA_x[0].theta_rho_z = ((dsge_param *)xp)[0].theta_rho_z;
   THETA_x[0].theta_D = ((dsge_param *)xp)[0].theta_D;
   THETA_x[0].theta_pi_star = ((dsge_param *)xp)[0].theta_pi_star;
   THETA_x[0].theta_r_star = ((dsge_param *)xp)[0].theta_r_star;
   THETA_x[0].theta_sigma_r = ((dsge_param *)xp)[0].theta_sigma_r;
   THETA_x[0].theta_sigma_z = ((dsge_param *)xp)[0].theta_sigma_z;
   THETA_x[0].theta_sigma_g = ((dsge_param *)xp)[0].theta_sigma_g;
   THETA_x[0].theta_sigma_pi = ((dsge_param *)xp)[0].theta_sigma_pi;
   THETA_x[0].theta_rho_gz = ((dsge_param *)xp)[0].theta_rho_gz;

   for (i= 0;i<17;i++)
{
  u[i] = gsl_rng_uniform(_gsl_r);
}
  THETA_new[0].theta_tau = u[0] * 2 * step_size - step_size + THETA_x[0].theta_tau;
  THETA_new[0].theta_beta = u[1] * 2 * step_size - step_size + THETA_x[0].theta_beta;
  THETA_new[0].theta_k = u[2] * 2 * step_size - step_size + THETA_x[0].theta_k;
  THETA_new[0].theta_psi1 = u[3] * 2 * step_size - step_size + THETA_x[0].theta_psi1;
  THETA_new[0].theta_psi2 = u[4] * 2 * step_size - step_size + THETA_x[0].theta_psi2;
  THETA_new[0].theta_rho_pi = u[5] * 2 * step_size - step_size + THETA_x[0].theta_rho_pi;
  THETA_new[0].theta_rho_r = u[6] * 2 * step_size - step_size + THETA_x[0].theta_rho_r;
  THETA_new[0].theta_rho_g = u[7] * 2 * step_size - step_size + THETA_x[0].theta_rho_g;
  THETA_new[0].theta_rho_z = u[8] * 2 * step_size - step_size + THETA_x[0].theta_rho_z;
  THETA_new[0].theta_D = u[9] * 2 * step_size - step_size + THETA_x[0].theta_D;
  THETA_new[0].theta_pi_star = u[10] * 2 * step_size - step_size + THETA_x[0].theta_pi_star;
  THETA_new[0].theta_r_star = u[11] * 2 * step_size - step_size + THETA_x[0].theta_r_star;
  THETA_new[0].theta_sigma_r = u[12] * 2 * step_size - step_size + THETA_x[0].theta_sigma_r;
  THETA_new[0].theta_sigma_z = u[13] * 2 * step_size - step_size + THETA_x[0].theta_sigma_z;
  THETA_new[0].theta_sigma_g = u[14] * 2 * step_size - step_size + THETA_x[0].theta_sigma_g;
  THETA_new[0].theta_sigma_pi = u[15] * 2 * step_size - step_size + THETA_x[0].theta_sigma_pi;
  THETA_new[0].theta_rho_gz = u[16] * 2 * step_size - step_size + THETA_x[0].theta_rho_gz;

  memcpy(xp, THETA_new, sizeof(dsge_param));
}

/*Print or, as I understand it, "output" function*/
void P_like(void *xp)
{
	int i;
/*(dsge_param *)xp;

printf("\n MLE theta_tau = %.8g \n", ((dsge_param *)xp)[0].theta_tau);
printf("MLE theta_beta = %.8g \n", ((dsge_param *)xp)[0].theta_beta);
printf("MLE theta_k = %.8g \n", ((dsge_param *)xp)[0].theta_k);
printf("MLE theta_psi1 = %.8g \n", ((dsge_param *)xp)[0].theta_psi1);
printf("MLE theta_psi2 = %.8g \n", ((dsge_param *)xp)[0].theta_psi2);
printf("MLE theta_rho_pi = %.8g \n", ((dsge_param *)xp)[0].theta_rho_pi);
printf("MLE theta_rho_r = %.8g \n", ((dsge_param *)xp)[0].theta_rho_r);
printf("MLE theta_rho_g = %.8g \n", ((dsge_param *)xp)[0].theta_rho_g);
printf("MLE theta_rho_z = %.8g \n", ((dsge_param *)xp)[0].theta_rho_z);
printf("MLE theta_D = %.8g \n", ((dsge_param *)xp)[0].theta_D);
printf("MLE theta_pi_star = %.8g \n", ((dsge_param *)xp)[0].theta_pi_star);
printf("MLE theta_r_star = %.8g \n", ((dsge_param *)xp)[0].theta_r_star);
printf("MLE theta_sigma_r = %.8g \n", ((dsge_param *)xp)[0].theta_sigma_r);
printf("MLE theta_sigma_z = %.8g \n", ((dsge_param *)xp)[0].theta_sigma_z);
printf("MLE theta_sigma_g = %.8g \n", ((dsge_param *)xp)[0].theta_sigma_g);
printf("MLE theta_sigma_pi = %.8g \n", ((dsge_param *)xp)[0].theta_sigma_pi);
printf("MLE theta_rho_gz = %.8g \n", ((dsge_param *)xp)[0].theta_rho_gz);
*/
for (i=0;i<17;i++)
	gsl_vector_set(SIM_ANN_MLE,i,*(&((dsge_param *)xp)[0].theta_tau+i));
/*	int data_T = 81;
	char inputfilename[14] = "DSGE_data2.txt"; 
	dsge_data_obj *data_X;
	data_X = dsge_read_in_data(inputfilename, data_T);
printf("MLE lglike(THETA_max, data_X, data_T) = %.8g \n", lglike_fn(((dsge_param *)xp), data_X, 0, data_T));*/
}


/*Simulated Annealing function takes the THETA[0] value as a starting point and using params defined above, 
plus the functions defined above. Output MLE is saved in the globally defined *SIM_ANN_MLE */
void SimAnneal(dsge_param *THETA_par, gsl_vector *v_THETA_MLE_par, dsge_data_obj *data_X, int data_T);


void SimAnneal(dsge_param *THETA_par, gsl_vector *v_THETA_MLE_par, dsge_data_obj *data_X, int data_T)
{
	clock_t start1=0,stop1=0; //Speed timing variables
	double Time=0;
	start1=clock();

//Setting these global variables allows me to avoid loading the data in E_like function every time it is called
	gl_data_X = data_X;
	gl_data_T = data_T;

  gsl_siman_solve(_gsl_r, THETA_par, E_like, S_like, M_like, P_like,
                  NULL, NULL, NULL, 
                  sizeof(dsge_param), params);
/*Note that the function above puts the final result of the MLE procedure in THETA_par vector
Note, that not entire THETA, which is a pointer to a pointer (matrix, allocated using malloc), was
given to SimmAnneal function as an argument, but only the first row of it
which is just a pointer (array) allocated as a pointer to a structure).*/


	stop1=clock();
	Time=(1.0*(stop1 - start1)) / (1.0*CLOCKS_PER_SEC);
	printf(" Simulated Annealing. \n Elapsed time = %g sec \n",Time);

	gsl_vector_memcpy (v_THETA_MLE_par, SIM_ANN_MLE);//copying the result in order to pass it to the main program

	//Outputting the results to *.txt files under the project folder
	{
unsigned int i_f;
FILE * f_f;
 
f_f = fopen("MLE_SimAnn.txt", "w");
for (i_f=0;i_f<(*v_THETA_MLE_par).size;i_f++)
      fprintf (f_f, "%.8g \n", gsl_vector_get(v_THETA_MLE_par,i_f) );
fclose (f_f);
	} 

	gsl_vector_free (SIM_ANN_MLE);

}
