#include "stdafx.h"
#include "mkl.h"



// Logical function that allows me to sort the eigenvalues for DGGES
int selctg(double *alphar, double *alphai, double* beta)
{
/*	printf("alphar = %.8g \n", *alphar);
	printf("alphai = %.8g \n", *alphai);
	printf("beta = %.8g \n", *beta);
*/
    int	res = 0;
    if(abs(*beta) < sqrt(pow(*alphar,2.0)+ pow(*alphai,2.0)))
		res = 1;
	return res;
}

// Matrix form conversion function from MKL to GSL of size (GSL_row x GSL_col)
void MKLtoGSL (double MKL_mat[], gsl_matrix * GSL_mat, int GSL_row, int GSL_col);

// Matrix form conversion function from GSL of size (GSL_row x GSL_col) to MKL
void GSLtoMKL (gsl_matrix * GSL_mat, double MKL_mat[], int GSL_row, int GSL_col);

// Function for printing GSL matrix to screen
void print_scr_gsl_mat (gsl_matrix * GSL_mat, int r, int c);

// Function for printing MKL matrix to screen
void print_scr_mkl_mat (double MKL_mat[], int r, int c);

// Function for setting the first n elements of an array a equal to the first n elements of array b
extern void eqarrays(double a[], double b[], int n);

//logPosterior:
double lglike_fn(dsge_param *THETA_par, dsge_data_obj *data_X_par, int MH_i_par, int data_T_par);

//logPrior:
double dsge_lgprior(dsge_param *THETA_par, int MH_i_par);

double lglike_fn(dsge_param *THETA_par, dsge_data_obj *data_X_par, int MH_i_par, int data_T_par)
{
	clock_t start1=0,stop1=0; //Speed timing variables
	double Time=0;
	start1=clock();


	int matsize = 8; //dimensionality of parameter space;
	int n=matsize, i,j;
	double zero_err = 0.0; // error used to check if solution exists by Sims's column spanning method

// DGGES parameters declaration (Generalized Schure QZ decomposition)
	char jobvsl='V';
	char jobvsr='V';
	char sort = 'S';
	int lda=matsize;
	int ldb=matsize;
    
	int sdim, info;
    
	double alphar[8];
	double alphai[8];
	double beta[8];

	double MKL_Q[8*8];
	double MKL_Z[8*8];

	int ldvsl=matsize;
	int ldvsr=matsize;
    
	double work[200];
	int lwork=200;
	int  bwork[8]; 

// DGEMM parameters declaration (C:=gemm_alpha*A*B+gemm_beta*C)
char transa, transb;
int gemm_m, gemm_n, gemm_k,	gemm_ldc, gemm_lda, gemm_ldb;  
double gemm_alpha, gemm_beta;

// DGESV parameters declaration (Matrix Inversion)
int gesv_n, gesv_nrhs, gesv_lda, gesv_ldb, gesv_info;
int ipiv[8];

// DGESVD parameters declaration (Singluar Value Decomposition)
char jobu, jobvt;
int gesvd_m, gesvd_n;
int gesvd_ldPi, gesvd_ldu, gesvd_ldvt;

// dgetrf parameters declaration (LU Decomposition)
int dgetrf_m, dgetrf_n, dgetrf_lda, dgetrf_info;

//Declaration of matricies needed for Linear Algebra
	double MKL_Q2[8*8];
	double MKL_Q1[8*8];
	gsl_matrix * GSL_Q = gsl_matrix_alloc(matsize,matsize);
	gsl_matrix * GSL_Q2 = gsl_matrix_alloc(matsize,matsize);
	gsl_matrix * GSL_Q1 = gsl_matrix_alloc(matsize,matsize);
	gsl_matrix * GSL_transmat = gsl_matrix_alloc(matsize,matsize);
	double MKL_Q2Pi[8*2];
	double MKL_Q1Pi[6*2];
	double MKL_QPsi[8*4];
	double MKL_QPsi_temp[8*4];
	
	double Q2Pitemp[2*2];
	double MKL_Q2Piinv[2*8];
	double transmat_temp[8*2];
	double MKL_transmat[]={1,0,0,0,0,0,0,0,
				0,1,0,0,0,0,0,0,
				0,0,1,0,0,0,0,0,
				0,0,0,1,0,0,0,0,
				0,0,0,0,1,0,0,0,
				0,0,0,0,0,1,0,0,
				0,0,0,0,0,0,0,0,
				0,0,0,0,0,0,0,0};/*The last two zerows on the main diagonal are needed to construct
								   MKL_transmat and MKL_T1 correctly*/
	double MKL_G0[8*8];
	double MKL_G0_inv[8*8];
	double MKL_T1[8*8];
	double MKL_T1_temp1[8*8];
	double MKL_T1_temp2[8*8];

	double MKL_TC_temp[8*8];
	double MKL_TC_temp2[8*8];
	double MKL_LamOm22[8*8];
	double MKL_LamOminvQ2[8*8];
	double MKL_TC[8*8];



	double MKL_D[1*2];
	double MKL_U[8*8];
	double MKL_Vt[2*2];
	double MKL_C[4];
	double MKL_T[64];
	double MKL_Wt[4*4];
	double MKL_D11inv[4];
	double MKL_temp1[4];
	double MKL_temp2[64];
	double MKL_Phi[8*4];
	double MKL_gamma0inv[64];
	double Id8by8[] = {1,0,0,0,0,0,0,0,
				0,1,0,0,0,0,0,0,
				0,0,1,0,0,0,0,0,
				0,0,0,1,0,0,0,0,
				0,0,0,0,1,0,0,0,
				0,0,0,0,0,1,0,0,
				0,0,0,0,0,0,1,0,
				0,0,0,0,0,0,0,1};
	double MKL_gamma1star[8*8];
	double MKL_gamma2star[8*4];
	double MKL_gamma3star[8*1];
	double MKL_Conststar[8*1];

//Kalman Filter Recursions variables declarations
	double state_tt1[8];
	double P_tt1[64];
	double f_tt1[3*3];
	double invf_tt1[3*3];
	double K_t[24];
    
	double y_t[3];

	double tempm1[64];
	double tempm2[64];
	double tempm3[8*4];
	double tempm4[3*8];
	double tempm6[3];
	double kernelm[1];
	double detf_tt1;
	int KF_i;
	double Id3by3[] = {1,0,0,
					   0,1,0,
					   0,0,1};

	
//initializing KF recursions variables
	double state_tt[] = {0,0,0,0,0,0,0,0};
	double P_tt[] = {0,0,0,0,0,0,0,0,
				0,0,0,0,0,0,0,0,
				0,0,0,0,0,0,0,0,
				0,0,0,0.35,0,0,0,0,
				0,0,0,0,0.24,0,0,0,
				0,0,0,0,0,0.19,0,0,
				0,0,0,0,0,0,0.05,0,
				0,0,0,0,0,0,0,0.74};

	double MKL_B[] = {1,0,0,
					  0,4,0,
					  0,0,4,
					  0,0,0,
					  0,0,0,
					  0,0,0,
					  0,0,0,
					  0,0,0};

  /*The following variables are "working" values of the parameters 
  in the current pass of the M-H loop declared here:*/
double theta_tau = THETA_par[MH_i_par].theta_tau, theta_k = THETA_par[MH_i_par].theta_k, 
theta_psi1 = THETA_par[MH_i_par].theta_psi1, theta_psi2 = THETA_par[MH_i_par].theta_psi2, theta_rho_pi = THETA_par[MH_i_par].theta_rho_pi,
theta_rho_r = THETA_par[MH_i_par].theta_rho_r, theta_rho_g = THETA_par[MH_i_par].theta_rho_g, theta_rho_z = THETA_par[MH_i_par].theta_rho_z, theta_D = THETA_par[MH_i_par].theta_D/*-THETA_par[MH_i_par].theta_rho_r*/,
theta_pi_star = THETA_par[MH_i_par].theta_pi_star, theta_r_star = THETA_par[MH_i_par].theta_r_star, theta_beta = THETA_par[MH_i_par].theta_beta /*0.992 *//*pow((1.0+(theta_r_star)/100.0),-0.25)*/,
theta_sigma_r = THETA_par[MH_i_par].theta_sigma_r, theta_sigma_z = THETA_par[MH_i_par].theta_sigma_z, theta_sigma_g = THETA_par[MH_i_par].theta_sigma_g,
theta_sigma_pi = THETA_par[MH_i_par].theta_sigma_pi, theta_rho_gz = THETA_par[MH_i_par].theta_rho_gz;

	double lglike = 0;  //Initializing the value of Log Likelihood
	lglike = dsge_lgprior(THETA_par, MH_i_par);//Comment out this line, if you do NOT
											   //want a prior in your model
/*Note that I use 'lglike' to denote a quantity which is best thought of as log(posterior)
even if the prior above is turned off, because the 'if' statement below is essnetially 
equivalent to lglike = lglike*prior, where prior is uniform on a "reasonable region" of the 
parameter space. So, either way, lglike represents log(posterior) here.*/

//Making sure that the current parameter values are not WACKY!
	if (theta_tau < 0.000001 || theta_k < 0.000001 || theta_psi1 < 1 || theta_psi2 < 0 || theta_rho_pi < 0 || theta_rho_r < 0 || theta_rho_g < -0.5 || 
	 theta_rho_z < 0 || theta_pi_star < 0 || theta_r_star < 0 || theta_sigma_r < 0 || theta_sigma_z < 0 || theta_D < -0.98 || theta_sigma_g < 0
	 || theta_sigma_pi < 0 || theta_rho_gz < -0.9 || theta_rho_r > 0.99 || theta_rho_pi > 0.99 || theta_rho_g > 0.98 || theta_rho_z > 0.99 || theta_rho_gz > 0.98
	 || theta_beta > 1.0)
	{
		//printf("WACKY parameters \n");
		lglike = -99999.00;
		return lglike;
	}

//don't forget that this 1-dimensional array corresponds to Column-major recorded matrix!!!
	double MKL_gamma1[]={0,0,0,0,0,0,0,0,
				0,0,0,0,0,0,0,0,
				0,0,theta_rho_r,0,0,0,0,0,
				0,0,0,theta_rho_pi,0,0,0,0,
				-1,theta_k,(1-theta_rho_r)*theta_psi2,0,0,0,1,0,
				0,-1,(1-theta_rho_r)*theta_psi1,0,0,0,0,1,
				0,0,0,0,theta_rho_g,0,0,0,
				0,0,0,0,0,theta_rho_z,0,0};

	//dgges function will give MKL_Omega the right value on the output
	double MKL_Omega[]={0,0,0,0,0,0,0,0,
				0,0,0,0,0,0,0,0,
				0,0,theta_rho_r,0,0,0,0,0,
				0,0,0,theta_rho_pi,0,0,0,0,
				-1,theta_k,(1-theta_rho_r)*theta_psi2,0,0,0,1,0,
				0,-1,(1-theta_rho_r)*theta_psi1,0,0,0,0,1,
				0,0,0,0,theta_rho_g,0,0,0,
				0,0,0,0,0,theta_rho_z,0,0};
	
    double MKL_gamma0[]={0,0,0,0,0,0,1,0,
				0,0,0,0,0,0,0,1,
				theta_tau,0,1,0,0,0,0,0,
				0,0,(1-theta_rho_r)*(1-theta_psi1),1,0,0,0,0,
				-1,0,0,0,0,0,0,0,
				- theta_tau,- theta_beta,0,0,0,0,0,0,
				-1,0,0,0,1,0,0,0,
				0,theta_k,(1-theta_rho_r)*theta_psi2,0,0,1,0,0};

//dgges function will give MKL_Lambda the right value on the output
	double MKL_Lambda[]={0,0,0,0,0,0,1,0,
				0,0,0,0,0,0,0,1,
				theta_tau,0,1,0,0,0,0,0,
				0,0,(1-theta_rho_r)*(1-theta_psi1),1,0,0,0,0,
				-1,0,0,0,0,0,0,0,
				- theta_tau,- theta_beta,0,0,0,0,0,0,
				-1,0,0,0,1,0,0,0,
				0,theta_k,(1-theta_rho_r)*theta_psi2,0,0,1,0,0};
 
// dgesv function will overwrite MKL_gamma0gesv with LU factorization
	double MKL_gamma0gesv[]={0,0,0,0,0,0,1,0,
				0,0,0,0,0,0,0,1,
				theta_tau,0,1,0,0,0,0,0,
				0,0,(1-theta_rho_r)*(1-theta_psi1),1,0,0,0,0,
				-1,0,0,0,0,0,0,0,
				- theta_tau,- theta_beta,0,0,0,0,0,0,
				-1,0,0,0,1,0,0,0,
				0,theta_k,(1-theta_rho_r)*theta_psi2,0,0,1,0,0};


	double MKL_Pi[] = {-1,theta_k,(1-theta_rho_r)*theta_psi2,0,0,0,1,0,
		0,-1,(1-theta_rho_r)*theta_psi1,0,0,0,0,1};

	double MKL_Psi[] = {0,0,1,0,0,0,0,0,
						0,0,0,0,1,0,0,0,
						0,0,0,0,0,1,0,0,
						0,0,0,1,0,0,0,0};
//dgemm function will give MKL_gamma2 the right value
	double MKL_gamma2[] = {0,0,1,0,0,0,0,0,
						   0,0,0,0,1,0,0,0,
						   0,0,0,0,0,1,0,0,
						   0,0,0,1,0,0,0,0};

	double MKL_gamma3[] = {0,0,theta_D+theta_rho_r,0,0,0,0,0};
	double MKL_Const[] = {0,0,0,0,0,0,0,0};

//Intercept of the measurement equation is not needed because I subtract it out from y_t below
//	double MKL_A[] = {0.0, theta_r_star, theta_r_star + theta_pi_star);

	double shock_Omega[] = {theta_sigma_r*theta_sigma_r, 0, 0, 0,
							0, theta_sigma_g*theta_sigma_g, theta_rho_gz*theta_sigma_z*theta_sigma_g, 0,
							0, theta_rho_gz*theta_sigma_z*theta_sigma_g, theta_sigma_z*theta_sigma_z, 0,
							0, 0, 0, theta_sigma_pi*theta_sigma_pi};
	
//*****************************************************************************
/*(QZ decomposition) Computes the generalized eigenvalues, Schur form, and the left 
  and/or right Schur vectors for a pair of nonsymmetric matrices*/

 dgges (&jobvsl, &jobvsr, &sort, selctg, &n, MKL_Lambda, &lda, MKL_Omega, &ldb,
		 &sdim, alphar, alphai, beta, MKL_Q, &ldvsl, MKL_Z, &ldvsr, work,
        &lwork,  bwork, &info );
//*****************************************************************************

 //check to make sure the dgges didn't produce incorrect results
 //need to introducing more elaborate error-handling later
if (!(info == 0) )
{	printf("DGGES Error, info = %d \n", info);
	getchar();
	lglike = -99999.00;
	return lglike;
}

/*printf("sdim = %d\n",sdim);
// printf("info = %d \n",info);
    printf("MKL_gamma1 =\n");
    print_scr_mkl_mat(MKL_gamma1, 8, 8);
    printf("MKL_gamma0 =\n");
    print_scr_mkl_mat(MKL_gamma0, 8, 8);
    printf("MKL_Omega =\n");
    print_scr_mkl_mat(MKL_Omega, 8, 8);
    printf("MKL_Lambda =\n");
    print_scr_mkl_mat(MKL_Lambda, 8, 8);
    printf("MKL_Q =\n");
    print_scr_mkl_mat(MKL_Q, 8, 8);
    printf("MKL_Z =\n");
    print_scr_mkl_mat(MKL_Z, 8, 8);
*/
 if (sdim > 6 )
{	printf("INDETERMINACY, because sdim = %d \n", sdim);

/*	printf ("theta_tau = %.8g \n theta_beta = %.8g \n theta_k = %.8g \n theta_psi1 = %.8g \n theta_psi2 = %.8g \n theta_rho_pi = %.8g \n theta_rho_r = %.8g \n"
			,theta_tau,theta_beta,theta_k,theta_psi1,theta_psi2,theta_rho_pi,theta_rho_r);
 

    printf("MKL_gamma1 =\n");
    print_scr_mkl_mat(MKL_gamma1, 8, 8);
    printf("MKL_gamma0 =\n");
    print_scr_mkl_mat(MKL_gamma0, 8, 8);
    printf("MKL_Omega =\n");
    print_scr_mkl_mat(MKL_Omega, 8, 8);
    printf("MKL_Lambda =\n");
    print_scr_mkl_mat(MKL_Lambda, 8, 8);
    printf("MKL_Q =\n");
    print_scr_mkl_mat(MKL_Q, 8, 8);
    printf("MKL_Z =\n");
    print_scr_mkl_mat(MKL_Z, 8, 8);

	getchar();
*/	lglike = -99999.00;
	return lglike;
}

//print_scr_mkl_mat(MKL_Q, matsize, matsize);
MKLtoGSL (MKL_Q, GSL_Q, matsize, matsize);
//print_scr_gsl_mat(GSL_Q, matsize, matsize);
gsl_matrix_transpose(GSL_Q); //GSL_Q = transpose (GSL_Q);
//print_scr_gsl_mat(GSL_Q, matsize, matsize);

//GSL_Q2 = Picking out the rows of GSL_Q that correspond to unstable roots
for(i= sdim;i<matsize;i++)
	{
	for(j=0;j<matsize;j++)
	gsl_matrix_set(GSL_Q2,i-sdim,j,gsl_matrix_get(GSL_Q,i,j));
	}
//print_scr_gsl_mat(GSL_Q2, matsize-sdim, matsize);
GSLtoMKL(GSL_Q2, MKL_Q2, matsize-sdim, matsize);

//GSL_Q1 = Picking out the rows of GSL_Q that correspond to stable roots
for(i= 0;i<sdim;i++)
	{
	for(j=0;j<matsize;j++)
	gsl_matrix_set(GSL_Q1,i,j,gsl_matrix_get(GSL_Q,i,j));
	}
//print_scr_gsl_mat(GSL_Q1, sdim, matsize);
GSLtoMKL(GSL_Q1, MKL_Q1, sdim, matsize);
gsl_matrix_free(GSL_Q);
gsl_matrix_free(GSL_Q2);
gsl_matrix_free(GSL_Q1);
//print_scr_mkl_mat(MKL_Q1, sdim, matsize);

//***** MKL_Q2Pi= MKL_Q2*MKL_Pi
transa = 'N'; transb = 'N';
gemm_m = matsize-sdim; gemm_n = 2; gemm_k = matsize; 
gemm_ldc = matsize-sdim; gemm_lda = matsize-sdim; gemm_ldb = matsize;  
gemm_alpha = 1.0; gemm_beta = 0.0;
dgemm(&transa, &transb, &gemm_m, &gemm_n, &gemm_k, &gemm_alpha, MKL_Q2, &gemm_lda, MKL_Pi, &gemm_ldb, &gemm_beta, MKL_Q2Pi, &gemm_ldc);
//print_scr_mkl_mat(MKL_Q2Pi, matsize-sdim, 2);

//***** MKL_Q1Pi= MKL_Q1*MKL_Pi
transa = 'N'; transb = 'N';
gemm_m = sdim; gemm_n = 2; gemm_k = matsize; 
gemm_ldc = gemm_m; gemm_lda = gemm_m; gemm_ldb = gemm_k;  
gemm_alpha = 1.0; gemm_beta = 0.0;
dgemm(&transa, &transb, &gemm_m, &gemm_n, &gemm_k, &gemm_alpha, MKL_Q1, &gemm_lda, MKL_Pi, &gemm_ldb, &gemm_beta, MKL_Q1Pi, &gemm_ldc);
//print_scr_mkl_mat(MKL_Q1Pi, sdim, 2);

//***** MKL_QPsi= MKL_Q2*MKL_Psi
transa = 'N'; transb = 'N';
gemm_m = matsize-sdim; gemm_n = 4; gemm_k = matsize; 
gemm_ldc = matsize-sdim; gemm_lda = matsize-sdim; gemm_ldb = matsize;  
gemm_alpha = 1.0; gemm_beta = 0.0;
dgemm(&transa, &transb, &gemm_m, &gemm_n, &gemm_k, &gemm_alpha, MKL_Q2, &gemm_lda, MKL_Psi, &gemm_ldb, &gemm_beta, MKL_QPsi, &gemm_ldc);
//print_scr_mkl_mat(MKL_QPsi, matsize-sdim, 4);


jobu = 'A'; jobvt ='A';
gesvd_m = matsize - sdim; gesvd_n = 2;
gesvd_ldPi = matsize - sdim; gesvd_ldu = matsize - sdim; gesvd_ldvt = 2;
//************Singular Value Decomposition******************************
dgesvd(&jobu, &jobvt, &gesvd_m, &gesvd_n, MKL_Q2Pi, &gesvd_ldPi, 
	   MKL_D, MKL_U, &gesvd_ldu, MKL_Vt, &gesvd_ldvt, work, &lwork, &info);
//**********************************************************************
 //check to make sure the dgesvd didn't produce incorrect results
 //need to introduce more elaborate error-handling later
if (!(info == 0) )
{	printf("DGESVD Error, info = %d \n", info);
	getchar();
	lglike = -99999.00;
	return lglike;
}
//printf("\n MKL_Vt =\n");
//print_scr_mkl_mat(MKL_Vt, 2, 2);

//********Checking the EXISTENCE condition (spaning equation (40) in Sims)**************
/*I set the intial value of the state variables in KF below to be all zero. Together with
condition 40, this implies that the current parameter vector doesn't lead to explosion*/ 
 if (sdim < 6 )
{	printf("May Be NO SOLUTION, because sdim = %d \n", sdim);
/*	getchar();
	lglike = -99999.00;
	return lglike;*/
}
//SVD:
jobu = 'A'; jobvt ='A';
gesvd_m = matsize - sdim; gesvd_n = 4;
gesvd_ldPi = matsize - sdim; gesvd_ldu = matsize - sdim; gesvd_ldvt = 4;
eqarrays(MKL_QPsi_temp, MKL_QPsi, matsize*4);
dgesvd(&jobu, &jobvt, &gesvd_m, &gesvd_n, MKL_QPsi_temp, &gesvd_ldPi, 
	   MKL_C, MKL_T, &gesvd_ldu, MKL_Wt, &gesvd_ldvt, work, &lwork, &info);
//check to make sure the dgesvd didn't produce incorrect results
	if (!(info == 0) )
	{	printf("DGESVD Error, info = %d \n", info);
		getchar();
		lglike = -99999.00;
		return lglike;
	}
	
//***** tempm2 = MKL_U * (MKL_U)'
transa = 'N'; transb = 'T';
gemm_m = matsize-sdim; gemm_n = matsize-sdim; gemm_k = 2; 
gemm_ldc = matsize-sdim; gemm_lda = matsize-sdim; gemm_ldb = matsize-sdim;  
gemm_alpha = 1.0; gemm_beta = 0.0;
dgemm(&transa, &transb, &gemm_m, &gemm_n, &gemm_k, &gemm_alpha, MKL_U, &gemm_lda, MKL_U, &gemm_ldb, &gemm_beta, tempm2, &gemm_ldc);

//***** ZERO = -tempm2*MKL_T + MKL_T
transa = 'N'; transb = 'N';
gemm_m = matsize-sdim; gemm_n = matsize-sdim; gemm_k = matsize-sdim; 
gemm_ldc = matsize-sdim; gemm_lda = matsize-sdim; gemm_ldb = matsize-sdim;  
gemm_alpha = -1.0; gemm_beta = 1.0;
dgemm(&transa, &transb, &gemm_m, &gemm_n, &gemm_k, &gemm_alpha, tempm2, &gemm_lda, MKL_T, &gemm_ldb, &gemm_beta, MKL_T, &gemm_ldc);

//Check if the resulting matrix is indeed ZERO matrix
for(i= 0;i<(matsize-sdim)*(matsize-sdim)-1;i++)
	zero_err =	zero_err + MKL_T[i]*MKL_T[i];
	zero_err = sqrt(zero_err);

if (zero_err > 0.000001*(matsize-sdim) )
	{
	printf("No solution, because zero_err = %.8g \n", zero_err);
	printf("sdim = %d\n",sdim);
/*
    printf("MKL_gamma1 =\n");
    print_scr_mkl_mat(MKL_gamma1, 8, 8);
    printf("MKL_gamma0 =\n");
    print_scr_mkl_mat(MKL_gamma0, 8, 8);
    printf("MKL_Omega =\n");
    print_scr_mkl_mat(MKL_Omega, 8, 8);
    printf("MKL_Lambda =\n");
    print_scr_mkl_mat(MKL_Lambda, 8, 8);
    printf("MKL_Q =\n");
    print_scr_mkl_mat(MKL_Q, 8, 8);
    printf("MKL_Z =\n");
    print_scr_mkl_mat(MKL_Z, 8, 8);

	getchar();
*/	lglike = -99999.00;
	return lglike;
	}


//********End Checking the EXISTENCE condition******************************************

/*Because we only have k=2 expectation errors, AND 2x1 MKL_D[] non-negative values 
are sorted in descending order, we have to cosider only the following 3 cases:
1. MKL_D[1]>0 - determinacy; 2. MKL_D[1] = 0 && MKL_D[0] > 0 - indeterminacy; 
3. MKL_D[1] = MKL_D[0] = 0 - impossible. It would mean that MKL_Q2Pi = 0 matrix
which means that there are no restrictions on eta_t at all. So, it is like 
2-degree indeterminacy case, as if there were no explosive roots after Shure Decomp.*/

if (MKL_D[1]<0.0000001 )
	{	
		printf("INDETERMINACY AT THETA[%d] DRAW, when sdim = %d, but MKL_D[1] = 0 \n", MH_i_par, sdim);
		lglike = -99999.00;
		return lglike;
	}

	MKL_D11inv[0] = 1.0 / MKL_D[0];
	MKL_D11inv[1] = 0.0;
	MKL_D11inv[2] = 0.0;
	MKL_D11inv[3] = 1.0 / MKL_D[1];
//printf("MKL_D11inv =\n");
//print_scr_mkl_mat(MKL_D11inv, 2, 2);

//***** MKL_temp1= - transpose(MKL_Vt)*MKL_D11inv
	transa = 'T'; transb = 'N';
	gemm_m = 2; gemm_n = 2; gemm_k = 2; 
	gemm_ldc = 2; gemm_lda = 2; gemm_ldb = 2;  
	gemm_alpha = -1.0; gemm_beta = 0.0;
dgemm(&transa, &transb, &gemm_m, &gemm_n, &gemm_k, &gemm_alpha, MKL_Vt, &gemm_lda, MKL_D11inv, &gemm_ldb, &gemm_beta, MKL_temp1, &gemm_ldc);
//printf("MKL_temp1 =\n");
//print_scr_mkl_mat(MKL_temp1, 2, 2);

//***** MKL_temp2 = MKL_temp1 * transpose(MKL_U)
	transa = 'N'; transb = 'T';
	gemm_m = 2; gemm_n = matsize - sdim; gemm_k = 2; 
	gemm_ldc = 2; gemm_lda = 2; gemm_ldb = gemm_n;  
	gemm_alpha = 1; gemm_beta = 0;
dgemm(&transa, &transb, &gemm_m, &gemm_n, &gemm_k, &gemm_alpha, MKL_temp1, &gemm_lda, MKL_U, &gemm_ldb, &gemm_beta, MKL_temp2, &gemm_ldc);
//print_scr_mkl_mat(MKL_U, 2, 2);
//printf("MKL_temp2 =\n");
//print_scr_mkl_mat(MKL_temp2, 2, matsize-sdim);

//******MKL_Phi = MKL_temp2 * MKL_QPsi
	transa = 'N'; transb = 'N';
	gemm_m = 2; gemm_n = 4; gemm_k = matsize - sdim;
	gemm_ldc = 2; gemm_lda = 2; gemm_ldb = matsize - sdim;
dgemm(&transa, &transb, &gemm_m, &gemm_n, &gemm_k, &gemm_alpha, MKL_temp2, &gemm_lda, MKL_QPsi, &gemm_ldb, &gemm_beta, MKL_Phi, &gemm_ldc);

//print_scr_mkl_mat(MKL_Phi, 2, 4);

//******MKL_gamma0inv = inv(MKL_gamma0)
eqarrays(MKL_gamma0inv, Id8by8, matsize*matsize);
gesv_n = matsize; gesv_nrhs = matsize; gesv_lda = matsize; gesv_ldb = matsize;
dgesv(&gesv_n,&gesv_nrhs,MKL_gamma0gesv,&gesv_lda,ipiv,MKL_gamma0inv,&gesv_ldb,&gesv_info);

if (!(gesv_info == 0) )
{	printf("DGESV Error, info = %d \n", gesv_info);
	getchar();
	lglike = -99999.00;
	return lglike;
}
//printf("MKL_gamma0inv  =\n");
//print_scr_mkl_mat(MKL_gamma0inv, matsize, matsize);


/*
//******MKL_gamma1star = MKL_gamma0inv * MKL_gamma1
	gemm_m = matsize; gemm_n = matsize; gemm_k = matsize;
	gemm_ldc = matsize; gemm_lda = matsize; gemm_ldb = matsize;
dgemm(&transa, &transb, &gemm_m, &gemm_n, &gemm_k, &gemm_alpha, MKL_gamma0inv, &gemm_lda, MKL_gamma1, &gemm_ldb, &gemm_beta, MKL_gamma1star, &gemm_ldc);
//printf("MKL_gamma1star =\n");
//print_scr_mkl_mat(MKL_gamma1star, matsize, matsize);*/

//**********************Calculating MKL_Gamma1star************************

//******Q2Pitemp = MKL_Vt * MKL_D11inv
	transa = 'T'; transb = 'N';
	gemm_m = 2; gemm_n = 2; gemm_k = 2; 
	gemm_ldc = gemm_m; gemm_lda = gemm_k; gemm_ldb = gemm_k;
	gemm_alpha = 1.0; gemm_beta = 0.0;
dgemm(&transa, &transb, &gemm_m, &gemm_n, &gemm_k, &gemm_alpha, MKL_Vt, &gemm_lda, MKL_D11inv, &gemm_ldb, &gemm_beta, Q2Pitemp, &gemm_ldc);

//******MKL_Q2Piinv = Q2Pitemp * MKL_U
	transa = 'N'; transb = 'T';
	gemm_m = 2; gemm_n = matsize - sdim; gemm_k = 2; 
	gemm_ldc = gemm_m; gemm_lda = gemm_m; gemm_ldb = gemm_n;
	gemm_alpha = 1.0; gemm_beta = 0.0;
dgemm(&transa, &transb, &gemm_m, &gemm_n, &gemm_k, &gemm_alpha, Q2Pitemp, &gemm_lda, MKL_U, &gemm_ldb, &gemm_beta, MKL_Q2Piinv, &gemm_ldc);

//******transmat_temp = MKL_Q1Pi * MKL_Q2Piinv
	transa = 'N'; transb = 'N';
	gemm_m = sdim; gemm_n = matsize - sdim; gemm_k = 2; 
	gemm_ldc = gemm_m; gemm_lda = gemm_m; gemm_ldb = gemm_k;
	gemm_alpha = 1.0; gemm_beta = 0.0;
dgemm(&transa, &transb, &gemm_m, &gemm_n, &gemm_k, &gemm_alpha, MKL_Q1Pi, &gemm_lda, MKL_Q2Piinv, &gemm_ldb, &gemm_beta, transmat_temp, &gemm_ldc);

//***Creating MKL_transmat from transmat_temp
for(i= (matsize-2)*matsize;i<(matsize-2)*matsize+sdim;i++)
	{
	MKL_transmat[i] = -transmat_temp[i-(matsize-2)*matsize];
	MKL_transmat[i+matsize] = -transmat_temp[i-(matsize-2)*matsize+sdim];
	}
/*
printf("transmat_temp =\n");
print_scr_mkl_mat(transmat_temp, sdim, 2);
printf("MKL_transmat =\n");
print_scr_mkl_mat(MKL_transmat, matsize, matsize);
*/
//******MKL_G0 = MKL_transmat * MKL_Lambda
	transa = 'N'; transb = 'N';
	gemm_m = matsize; gemm_n = matsize; gemm_k = matsize; 
	gemm_ldc = gemm_m; gemm_lda = gemm_m; gemm_ldb = gemm_k;
	gemm_alpha = 1.0; gemm_beta = 0.0;
dgemm(&transa, &transb, &gemm_m, &gemm_n, &gemm_k, &gemm_alpha, MKL_transmat, &gemm_lda, MKL_Lambda, &gemm_ldb, &gemm_beta, MKL_G0, &gemm_ldc);

MKL_G0[matsize*matsize-1] = 1.0; MKL_G0[(matsize-1)*matsize-2] = 1.0;
/*printf("MKL_G0  =\n");
print_scr_mkl_mat(MKL_G0, matsize, matsize);*/

//******MKL_G0_inv = inv(MKL_G0)
eqarrays(MKL_G0_inv, Id8by8, matsize*matsize);
gesv_n = matsize; gesv_nrhs = matsize; gesv_lda = matsize; gesv_ldb = matsize;
dgesv(&gesv_n,&gesv_nrhs,MKL_G0,&gesv_lda,ipiv,MKL_G0_inv,&gesv_ldb,&gesv_info);

if (!(gesv_info == 0) )
{	printf("DGESV Error, info = %d \n", gesv_info);
	getchar();
	lglike = -99999.00;
	return lglike;
}
/*printf("MKL_G0_inv  =\n");
print_scr_mkl_mat(MKL_G0_inv, matsize, matsize);*/

//******MKL_T1 = MKL_transmat * MKL_Omega
	transa = 'N'; transb = 'N';
	gemm_m = matsize; gemm_n = matsize; gemm_k = matsize; 
	gemm_ldc = gemm_m; gemm_lda = gemm_m; gemm_ldb = gemm_k;
	gemm_alpha = 1.0; gemm_beta = 0.0;
dgemm(&transa, &transb, &gemm_m, &gemm_n, &gemm_k, &gemm_alpha, MKL_transmat, &gemm_lda, MKL_Omega, &gemm_ldb, &gemm_beta, MKL_T1, &gemm_ldc);
/*printf("MKL_T1  =\n");
print_scr_mkl_mat(MKL_T1, matsize, matsize);*/

//******MKL_T1_temp1 = MKL_G0_inv * MKL_T1
	transa = 'N'; transb = 'N';
	gemm_m = matsize; gemm_n = matsize; gemm_k = matsize; 
	gemm_ldc = gemm_m; gemm_lda = gemm_m; gemm_ldb = gemm_k;
	gemm_alpha = 1.0; gemm_beta = 0.0;
dgemm(&transa, &transb, &gemm_m, &gemm_n, &gemm_k, &gemm_alpha, MKL_G0_inv, &gemm_lda, MKL_T1, &gemm_ldb, &gemm_beta, MKL_T1_temp1, &gemm_ldc);

//******MKL_T1_temp2 = MKL_Z * MKL_T1_temp1
	transa = 'N'; transb = 'N';
	gemm_m = matsize; gemm_n = matsize; gemm_k = matsize; 
	gemm_ldc = gemm_m; gemm_lda = gemm_m; gemm_ldb = gemm_k;
	gemm_alpha = 1.0; gemm_beta = 0.0;
dgemm(&transa, &transb, &gemm_m, &gemm_n, &gemm_k, &gemm_alpha, MKL_Z, &gemm_lda, MKL_T1_temp1, &gemm_ldb, &gemm_beta, MKL_T1_temp2, &gemm_ldc);

//******MKL_gamma1star = MKL_T1_temp2 * transpose(MKL_Z)
	transa = 'N'; transb = 'T';
	gemm_m = matsize; gemm_n = matsize; gemm_k = matsize; 
	gemm_ldc = gemm_m; gemm_lda = gemm_m; gemm_ldb = gemm_n;
	gemm_alpha = 1.0; gemm_beta = 0.0;
dgemm(&transa, &transb, &gemm_m, &gemm_n, &gemm_k, &gemm_alpha, MKL_T1_temp2, &gemm_lda, MKL_Z, &gemm_ldb, &gemm_beta, MKL_gamma1star, &gemm_ldc);
/*printf("MKL_gamma1star =\n");
print_scr_mkl_mat(MKL_gamma1star, matsize, matsize);*/


//**********************Calculating MKL_gamma2star************************

//******MKL_gamma2 = MKL_Psi + MKL_Pi * MKL_Phi
	transa = 'N'; transb = 'N';
	gemm_m = matsize; gemm_n = 4; gemm_k = 2; 
	gemm_ldc = matsize; gemm_lda = matsize; gemm_ldb = 2;  
	gemm_alpha = 1; gemm_beta = 1;
dgemm(&transa, &transb, &gemm_m, &gemm_n, &gemm_k, &gemm_alpha, MKL_Pi, &gemm_lda, MKL_Phi, &gemm_ldb, &gemm_beta, MKL_gamma2, &gemm_ldc);
//printf("MKL_gamma2 =\n");
//print_scr_mkl_mat(MKL_gamma2, matsize, 4);

//******MKL_gamma2star = MKL_gamma0inv * MKL_gamma2
	transa = 'N'; transb = 'N';
	gemm_m = matsize; gemm_n = 4; gemm_k = matsize; 
	gemm_ldc = matsize; gemm_lda = matsize; gemm_ldb = matsize;  
	gemm_alpha = 1.0; gemm_beta = 0.0;
dgemm(&transa, &transb, &gemm_m, &gemm_n, &gemm_k, &gemm_alpha, MKL_gamma0inv, &gemm_lda, MKL_gamma2, &gemm_ldb, &gemm_beta, MKL_gamma2star, &gemm_ldc);
//printf("MKL_gamma2star =\n");
//print_scr_mkl_mat(MKL_gamma2star, matsize, 4);


if ((MKL_Const[0]!=0)||(MKL_Const[1]!=0)||(MKL_Const[2]!=0)||(MKL_Const[3]!=0))
{
//**********************Calculating MKL_Conststar************************
//******MKL_TC_temp = MKL_transmat * transpose (MKL_Q)
	transa = 'N'; transb = 'T';
	gemm_m = matsize; gemm_n = matsize; gemm_k = matsize; 
	gemm_ldc = matsize; gemm_lda = matsize; gemm_ldb = matsize;  
	gemm_alpha = 1.0; gemm_beta = 0.0;
dgemm(&transa, &transb, &gemm_m, &gemm_n, &gemm_k, &gemm_alpha, MKL_transmat, &gemm_lda, MKL_Q, &gemm_ldb, &gemm_beta, MKL_TC_temp, &gemm_ldc);
//printf("MKL_TC_temp =\n");
//print_scr_mkl_mat(MKL_TC_temp, matsize, matsize);

//***Creating MKL_LamOm22 = Omega22 - Lambda22
for(i= 0;i<matsize-sdim;i++)
	{
	for(j= 0;j<matsize-sdim;j++)
		MKL_LamOm22[i*(matsize-sdim)+j] = MKL_Lambda[sdim*matsize+sdim + (i*matsize+j)] - MKL_Omega[sdim*matsize+sdim + (i*matsize+j)];
	}

//printf("MKL_LamOm22 (before inversion) =\n");
//print_scr_mkl_mat(MKL_LamOm22, matsize-sdim, matsize-sdim);

//******MKL_LamOm22 = inv(MKL_LamOm22)
dgetrf_n = matsize-sdim; dgetrf_m = matsize-sdim; dgetrf_lda = matsize-sdim;;
//***LU decomposition of MKL_LamOm22
dgetrf(&dgetrf_m, &dgetrf_n, MKL_LamOm22, &dgetrf_lda, ipiv, &dgetrf_info);
//***Inverse of LU-form
dgetri (&dgetrf_n, MKL_LamOm22, &dgetrf_lda, ipiv, work, &lwork, &info);

if (!(info == 0) )
{	printf("dgetri Error, info = %d \n", info);
	getchar();
	lglike = -99999.00;
	return lglike;
}
//printf("MKL_LamOm22 (after inversion) =\n");
//print_scr_mkl_mat(MKL_LamOm22, matsize-sdim, matsize-sdim);

//******MKL_LamOminvQ2 = MKL_LamOm22 * MKL_Q2
	transa = 'N'; transb = 'N';
	gemm_m = matsize-sdim; gemm_n = matsize; gemm_k = matsize-sdim; 
	gemm_ldc = gemm_m; gemm_lda = gemm_m; gemm_ldb = gemm_k;  
	gemm_alpha = 1.0; gemm_beta = 0.0;
dgemm(&transa, &transb, &gemm_m, &gemm_n, &gemm_k, &gemm_alpha, MKL_LamOm22, &gemm_lda, MKL_Q2, &gemm_ldb, &gemm_beta, MKL_LamOminvQ2, &gemm_ldc);

//*****Filling in the unstable rows of MKL_TC_temp with MKL_LamOminvQ2 elements
for(i= 0;i<matsize;i++)
	{
	for(j= sdim;j<matsize;j++)
		MKL_TC_temp[i*matsize+j] = MKL_LamOminvQ2[i*(matsize-sdim)+ j-sdim];
	}
//printf("MKL_TC_temp =\n");
//print_scr_mkl_mat(MKL_TC_temp, matsize, matsize);

//******MKL_TC_temp2 = MKL_G0_inv * MKL_TC_temp
	transa = 'N'; transb = 'N';
	gemm_m = matsize; gemm_n = matsize; gemm_k = matsize; 
	gemm_ldc = gemm_m; gemm_lda = gemm_m; gemm_ldb = gemm_k;  
	gemm_alpha = 1.0; gemm_beta = 0.0;
dgemm(&transa, &transb, &gemm_m, &gemm_n, &gemm_k, &gemm_alpha, MKL_G0_inv, &gemm_lda, MKL_TC_temp, &gemm_ldb, &gemm_beta, MKL_TC_temp2, &gemm_ldc);

//******MKL_TC = MKL_Z * MKL_TC_temp2
	transa = 'N'; transb = 'N';
	gemm_m = matsize; gemm_n = matsize; gemm_k = matsize; 
	gemm_ldc = gemm_m; gemm_lda = gemm_m; gemm_ldb = gemm_k;  
	gemm_alpha = 1.0; gemm_beta = 0.0;
dgemm(&transa, &transb, &gemm_m, &gemm_n, &gemm_k, &gemm_alpha, MKL_Z, &gemm_lda, MKL_TC_temp2, &gemm_ldb, &gemm_beta, MKL_TC, &gemm_ldc);

//******MKL_Conststar = MKL_TC * MKL_Const
	transa = 'N'; transb = 'N';
	gemm_m = matsize; gemm_n = 1; gemm_k = matsize; 
	gemm_ldc = gemm_m; gemm_lda = gemm_m; gemm_ldb = gemm_k;  
	gemm_alpha = 1.0; gemm_beta = 0.0; 
dgemm(&transa, &transb, &gemm_m, &gemm_n, &gemm_k, &gemm_alpha, MKL_TC, &gemm_lda, MKL_Const, &gemm_ldb, &gemm_beta, MKL_Conststar, &gemm_ldc);
//printf("MKL_Conststar =\n");
//print_scr_mkl_mat(MKL_Conststar, matsize, 1);
}

//**********************Calculating MKL_gamma3star************************
//******MKL_gamma3star = MKL_gamma0inv * MKL_gamma3
	transa = 'N'; transb = 'N';
	gemm_m = matsize; gemm_n = 1; gemm_k = matsize;
	gemm_ldc = gemm_m; gemm_lda = gemm_m; gemm_ldb = gemm_k;  
	gemm_alpha = 1.0; gemm_beta = 0.0; 
dgemm(&transa, &transb, &gemm_m, &gemm_n, &gemm_k, &gemm_alpha, MKL_gamma0inv, &gemm_lda, MKL_gamma3, &gemm_ldb, &gemm_beta, MKL_gamma3star, &gemm_ldc);
//printf("MKL_gamma3star =\n");
//print_scr_mkl_mat(MKL_gamma3star, matsize, 1);


//****************************************************************************************
//***********Linear Algebra has ended. Now Computing Kalman Filter Recursions*************
//****************************************************************************************


for (KF_i= 0;KF_i<data_T_par;KF_i++)
	{
//First, set the data vector (note that I subtracted out the intercept term for convinience of coding):
		y_t[0] = data_X_par[KF_i].data_gdp;
		y_t[1] = data_X_par[KF_i].data_pi - theta_pi_star;
		y_t[2] = data_X_par[KF_i].data_r - (theta_r_star + theta_pi_star);

eqarrays(state_tt1, MKL_gamma3star, matsize);
//******state_tt1 = MKL_gamma1star * state_tt + data_X_par[KF_i].data_delta*state_tt1
	transa = 'N'; transb = 'N';
	gemm_m = matsize; gemm_n = 1; gemm_k = matsize;
	gemm_ldc = gemm_m; gemm_lda = gemm_m; gemm_ldb = gemm_k;
	gemm_alpha = 1.0; gemm_beta = data_X_par[KF_i].data_delta;
dgemm(&transa, &transb, &gemm_m, &gemm_n, &gemm_k, &gemm_alpha, MKL_gamma1star, &gemm_lda, state_tt, &gemm_ldb, &gemm_beta, state_tt1, &gemm_ldc);

//******tempm1 = MKL_gamma1star * P_tt
gemm_n = matsize;
gemm_beta = 0.0;
dgemm(&transa, &transb, &gemm_m, &gemm_n, &gemm_k, &gemm_alpha, MKL_gamma1star, &gemm_lda, P_tt, &gemm_ldb, &gemm_beta, tempm1, &gemm_ldc);

//******P_tt1 =  tempm1 * transpose(MKL_gamma1star)
transb = 'T';
gemm_ldb = gemm_n;
dgemm(&transa, &transb, &gemm_m, &gemm_n, &gemm_k, &gemm_alpha, tempm1, &gemm_lda, MKL_gamma1star, &gemm_ldb, &gemm_beta, P_tt1, &gemm_ldc);

//******tempm3 =  MKL_gamma2star * shock_Omega
	transa = 'N'; transb = 'N';
	gemm_m = matsize; gemm_n = 4; gemm_k = 4;
	gemm_ldc = gemm_m; gemm_lda = gemm_m; gemm_ldb = gemm_k;
dgemm(&transa, &transb, &gemm_m, &gemm_n, &gemm_k, &gemm_alpha, MKL_gamma2star, &gemm_lda, shock_Omega, &gemm_ldb, &gemm_beta, tempm3, &gemm_ldc);

//******P_tt1 =  tempm3 * transpose(MKL_gamma2star) + P_tt1
transb = 'T';
gemm_n = matsize;
gemm_ldb = gemm_n;
gemm_beta = 1.0;
dgemm(&transa, &transb, &gemm_m, &gemm_n, &gemm_k, &gemm_alpha, tempm3, &gemm_lda, MKL_gamma2star, &gemm_ldb, &gemm_beta, P_tt1, &gemm_ldc);
//print_scr_mkl_mat(P_tt1, 8, 8);

//******tempm4 =  MKL_B * P_tt1
	transa = 'N'; transb = 'N';
	gemm_m = 3; gemm_n = matsize; gemm_k = matsize;
	gemm_ldc = gemm_m; gemm_lda = gemm_m; gemm_ldb = gemm_k;
	gemm_beta = 0.0;
dgemm(&transa, &transb, &gemm_m, &gemm_n, &gemm_k, &gemm_alpha, MKL_B, &gemm_lda, P_tt1, &gemm_ldb, &gemm_beta, tempm4, &gemm_ldc);

//******f_tt1 =  tempm4 * transpose(MKL_B)
transb = 'T';
gemm_n = 3;
gemm_ldb = gemm_n;
dgemm(&transa, &transb, &gemm_m, &gemm_n, &gemm_k, &gemm_alpha, tempm4, &gemm_lda, MKL_B, &gemm_ldb, &gemm_beta, f_tt1, &gemm_ldc);

/*****************Break in the KF recursions evaluation to calculate the likelihood*/
// Determinant of 3x3 invf_tt1: (Note that in general LU decomposition can be used to get the determinant
// by simply multiplying out the diagonal elements of L and U matricies)
detf_tt1 = abs(f_tt1[0]*(f_tt1[4]*f_tt1[8] - f_tt1[7]*f_tt1[5])-
			  f_tt1[1]*(f_tt1[3]*f_tt1[8] - f_tt1[6]*f_tt1[5])+
			  f_tt1[2]*(f_tt1[3]*f_tt1[7] - f_tt1[6]*f_tt1[4]));
//printf("detf_tt1(%d) = %.8g \n", KF_i, detf_tt1);

//******invf_tt1 = inv(f_tt1)
eqarrays(invf_tt1, Id3by3, 9);
gesv_n = 3; gesv_nrhs = 3; gesv_lda = 3; gesv_ldb = 3;
dgesv(&gesv_n,&gesv_nrhs,f_tt1,&gesv_lda,ipiv,invf_tt1,&gesv_ldb,&gesv_info);
//print_scr_mkl_mat(f_tt1, 3, 3);
//print_scr_mkl_mat(invf_tt1, 3, 3);

//******y_t = -MKL_B * state_tt1 + y_t 
	transa = 'N'; transb = 'N';
	gemm_m = 3; gemm_n = 1; gemm_k = matsize;
	gemm_ldc = gemm_m; gemm_lda = gemm_m; gemm_ldb = gemm_k;
	gemm_alpha = -1.0; gemm_beta = 1.0;
dgemm(&transa, &transb, &gemm_m, &gemm_n, &gemm_k, &gemm_alpha, MKL_B, &gemm_lda, state_tt1, &gemm_ldb, &gemm_beta, y_t, &gemm_ldc);

//******tempm6 = transpose(y_t) * invf_tt1
	transa = 'T'; transb = 'N';
	gemm_m = 1; gemm_n = 3; gemm_k = 3;
	gemm_ldc = gemm_m; gemm_lda = gemm_k; gemm_ldb = gemm_k;
	gemm_alpha = 1.0; gemm_beta = 0.0;
dgemm(&transa, &transb, &gemm_m, &gemm_n, &gemm_k, &gemm_alpha, y_t, &gemm_lda, invf_tt1, &gemm_ldb, &gemm_beta, tempm6, &gemm_ldc);

//******kernelm = tempm6 * y_t
	transa = 'N'; transb = 'N';
	gemm_m = 1; gemm_n = 1; gemm_k = 3;
	gemm_ldc = gemm_m; gemm_lda = gemm_m; gemm_ldb = gemm_k;
	gemm_alpha = 1.0; gemm_beta = 0.0;
dgemm(&transa, &transb, &gemm_m, &gemm_n, &gemm_k, &gemm_alpha, tempm6, &gemm_lda, y_t, &gemm_ldb, &gemm_beta, kernelm, &gemm_ldc);
//print_scr_mkl_mat(kernelm, 1, 1);

//Loglikelihood calculation:
if(KF_i > 5)
lglike = lglike - 0.5*(log(detf_tt1) + kernelm[0]); //The 'if' statement ensures that the effect of initial setting of P_tt is minimal
/*printf("lglike(%d) = %.8g \n", KF_i, lglike);
printf("\n");
printf("\n");
printf("\n");*/

/*****************Continue with KF recursions evaluation*/
//******tempm4 =  P_tt1 * transpose(MKL_B)
	transa = 'N'; transb = 'T';
	gemm_m = matsize; gemm_n = 3; gemm_k = matsize;
	gemm_ldc = gemm_m; gemm_lda = gemm_m; gemm_ldb = gemm_n;
	gemm_alpha = 1.0; gemm_beta = 0.0;
dgemm(&transa, &transb, &gemm_m, &gemm_n, &gemm_k, &gemm_alpha, P_tt1, &gemm_lda, MKL_B, &gemm_ldb, &gemm_beta, tempm4, &gemm_ldc);

//******K_t = tempm4 * invf_tt1
	transa = 'N'; transb = 'N';
	gemm_m = matsize; gemm_n = 3; gemm_k = 3;
	gemm_ldc = gemm_m; gemm_lda = gemm_m; gemm_ldb = gemm_k;
	gemm_alpha = 1.0; gemm_beta = 0.0;
dgemm(&transa, &transb, &gemm_m, &gemm_n, &gemm_k, &gemm_alpha, tempm4, &gemm_lda, invf_tt1, &gemm_ldb, &gemm_beta, K_t, &gemm_ldc);

//******state_tt = K_t * y_t + state_tt1
eqarrays(state_tt, state_tt1, matsize);
	transa = 'N'; transb = 'N';
	gemm_m = matsize; gemm_n = 1; gemm_k = 3;
	gemm_ldc = gemm_m; gemm_lda = gemm_m; gemm_ldb = gemm_k;
	gemm_alpha = 1.0; gemm_beta = 1.0;
dgemm(&transa, &transb, &gemm_m, &gemm_n, &gemm_k, &gemm_alpha, K_t, &gemm_lda, y_t, &gemm_ldb, &gemm_beta, state_tt, &gemm_ldc);

//******tempm1 = -K_t * MKL_B + Identity
eqarrays(tempm1, Id8by8, matsize*matsize);
	transa = 'N'; transb = 'N';
	gemm_m = matsize; gemm_n = 8; gemm_k = 3;
	gemm_ldc = gemm_m; gemm_lda = gemm_m; gemm_ldb = gemm_k;
	gemm_alpha = -1.0; gemm_beta = 1.0;
dgemm(&transa, &transb, &gemm_m, &gemm_n, &gemm_k, &gemm_alpha, K_t, &gemm_lda, MKL_B, &gemm_ldb, &gemm_beta, tempm1, &gemm_ldc);

//******P_tt = tempm1 * P_tt1
gemm_k = 8;
gemm_ldb = gemm_k;
gemm_alpha = 1.0; gemm_beta = 0.0;
dgemm(&transa, &transb, &gemm_m, &gemm_n, &gemm_k, &gemm_alpha, tempm1, &gemm_lda, P_tt1, &gemm_ldb, &gemm_beta, P_tt, &gemm_ldc);

//NOTE: Overwriting is NOT necessary, b/c Kalman Filter takes care of it automatiacally (see Harvey p.109)

	}
	
	stop1=clock();
	Time=(1.0*(stop1 - start1)) / (1.0*CLOCKS_PER_SEC);
	//printf(" End of likelihood calculation. \n Elapsed time = %.8g sec \n",Time);

  return lglike;
}


double dsge_lgprior(dsge_param *THETA_par, int MH_i_par)
{
double theta_tau = THETA_par[MH_i_par].theta_tau, theta_k = THETA_par[MH_i_par].theta_k, 
theta_psi1 = THETA_par[MH_i_par].theta_psi1, theta_psi2 = THETA_par[MH_i_par].theta_psi2, theta_rho_pi = THETA_par[MH_i_par].theta_rho_pi,
theta_rho_r = THETA_par[MH_i_par].theta_rho_r, theta_rho_g = THETA_par[MH_i_par].theta_rho_g, theta_rho_z = THETA_par[MH_i_par].theta_rho_z, theta_D = THETA_par[MH_i_par].theta_D /*-THETA_par[MH_i_par].theta_rho_r*/,
theta_pi_star = THETA_par[MH_i_par].theta_pi_star, theta_r_star = THETA_par[MH_i_par].theta_r_star, theta_beta = THETA_par[MH_i_par].theta_beta /*0.992*/ /*pow((1.0+(theta_r_star+6.0)/100.0),-0.25)*/,
theta_sigma_r = THETA_par[MH_i_par].theta_sigma_r, theta_sigma_z = THETA_par[MH_i_par].theta_sigma_z, theta_sigma_g = THETA_par[MH_i_par].theta_sigma_g,
theta_sigma_pi = THETA_par[MH_i_par].theta_sigma_pi, theta_rho_gz = THETA_par[MH_i_par].theta_rho_gz;

double a, b, prior = 0;
//These are kernels of Gamma, Normal and IG for volatilities

a = 14.332, b = 0.037;
prior = prior + (a-1)*log(theta_tau) - theta_tau/(b);

a = 0.99, b = 0.005;
prior = prior - 0.5*(pow((theta_beta-a)/b,2.0));


a = 6.25, b = 0.08;
prior = prior + (a-1)*log(theta_k) - theta_k/(b);

a = 1.8, b = 0.4;
prior = prior - 0.5*(pow((theta_psi1-a)/b,2.0));

a = 0.3, b = 0.1;
prior = prior - 0.5*(pow((theta_psi2-a)/b,2.0));

a = 0.87, b = 0.1;
prior = prior - 0.5*(pow((theta_rho_pi-a)/b,2.0));

a = 0.5, b = 0.2;
prior = prior - 0.5*(pow((theta_rho_r-a)/b,2.0));

a = 0.7, b = 0.1;
prior = prior - 0.5*(pow((theta_rho_g-a)/b,2.0));

a = 0.7, b = 0.1;
prior = prior - 0.5*(pow((theta_rho_z-a)/b,2.0));

a = 0.0, b = 0.3;
prior = prior - 0.5*(pow((theta_D-a)/b,2.0));

a = 4.0, b = 2.0;
prior = prior - 0.5*(pow((theta_pi_star-a)/b,2.0));

a = 2.0, b = 1.0;
prior = prior - 0.5*(pow((theta_r_star-a)/b,2.0));

a = 5.61, b = 0.571;
prior = prior - (a+1)*log(theta_sigma_g) - 1/(b*theta_sigma_g);//See .xls file used to go from mean and variance
															   // to parameres and b (used "solver")
a = 5.754, b = 0.6786;
prior = prior - (a+1)*log(theta_sigma_r) - 1/(b*theta_sigma_r);

a = 5.698, b = 0.2128;
prior = prior - (a+1)*log(theta_sigma_z) - 1/(b*theta_sigma_z);

a = 2.69, b = 1.18;
prior = prior - (a+1)*log(theta_sigma_pi) - 1/(b*theta_sigma_pi);

a = 0.0, b = 0.4;
prior = prior - 0.5*(pow((theta_rho_gz-a)/b,2.0));

return prior;
}