#include "stdafx.h"


extern double lglike_fn(dsge_param *THETA_par, dsge_data_obj *data_X_par, int MH_i_par, int data_T_par);
double my_dfridr(const gsl_function *f, double x, double h, double *err);

//In this file I create "gsl_function" type of likelihood function lglike_fn needed for numerical derivatives
//(both Gradient and Hessian) and BFGS maximization method
double DSGE_like_f (const gsl_vector *v_THETA, void *p);
double partial_function(double x, void *p);
void DSGE_like_df (const gsl_vector *v_THETA, void *p, gsl_vector *gradient);
void DSGE_like_fdf (const gsl_vector *v_THETA, void *p, double *f, gsl_vector *gradient);

extern double h; //Reference to global derivative step size h defined on the next line

double DSGE_like_f (const gsl_vector *v_THETA, void *p)
{	
	int i;
	double res;
	struct DSGE_like_params * params  = (struct DSGE_like_params *)(p);
	dsge_param *THETA = (dsge_param*) malloc( (unsigned) sizeof(dsge_param));
for (i=0;i<17;i++)
	*(&THETA[0].theta_tau+i) = gsl_vector_get(v_THETA,i);

res = -lglike_fn(THETA, (*params).data_X, 0, (*params).data_T); 
free (THETA);
return res;
}

double partial_function(double x, void *p)
{
struct DSGE_like_params * params  = (struct DSGE_like_params *)(p);
dsge_param *THETA = (dsge_param*) malloc( (unsigned) sizeof(dsge_param));
	int i;
	double res;
	for(i= 0;i<(*params).dim;i++)
		{
		*(&THETA[0].theta_tau +i) = gsl_vector_get((*params).v_x,i);
		if (i == (*params).partial)
		*(&THETA[0].theta_tau +i) = x;
		}

res = -lglike_fn(THETA, (*params).data_X, 0, (*params).data_T);
free (THETA);
return res;
}

void DSGE_like_df (const gsl_vector *v_THETA, void *p, gsl_vector *gradient)
{
	int i,j;
	struct DSGE_like_params * params  = (struct DSGE_like_params *)(p);
	gsl_function partial_f;
	double *err = (double *) malloc( (unsigned) sizeof(double));;
//	double h = 0.00000001;
	double x;

for (i=0;i<17;i++)
	{
	gsl_vector_set((*params).v_x, i,gsl_vector_get(v_THETA,i));//Copy all the values of the argument into parameter structure (*params)
	}

for(j= 0;j<(*params).dim;j++)
	{
	(*params).partial = j;
	x = gsl_vector_get(v_THETA,j);
	partial_f.function = &partial_function;
	partial_f.params = (void *) params;

	gsl_vector_set(gradient,j, my_dfridr(&partial_f, x, h, err));
	}
	
	free(err);
}

void DSGE_like_fdf (const gsl_vector *v_THETA, void *p, double *f, gsl_vector *gradient)
{
	*f = DSGE_like_f(v_THETA, p);
	DSGE_like_df (v_THETA, p, gradient);
}