/******************************************************************/
/* THIS FUNCTION CALCULATES KERNEL (PDF W/O NORMALIZING CONSTANT) */
/* OF N-DIMENSIONAL STUDENT-T DISTRIBUTION                        */
/*                                                                */
/* OUTPUT: (double) pdf value                                     */
/*                                                                */
/* INPUT: mean, INV(variance), degrees of freedom, # dimensions,  */
/*         array point at which kernel is calculated              */
/*                                                                */
/* NOTE: INVERSE(!!!) OF THE COVARIANCE MATRIX IS USED HERE!!     */
/*       THAT IS BECAUSE INVERSE IS USUALLY AVAILABLE IN TAILORED */
/*       APPROACH EVEN BEFORE THE VARIANCE IS AVAILABLE.          */
/*       SO WE CAN AVOID UNNECESSARY INVERSE COMPUTATION          */
/*       WHICH IS TOO COSTLY INSIDE M-H LOOP                      */
/******************************************************************/

#include <stdio.h>
#include <math.h>
#include <stdlib.h>

/*Note that blas_dgemm declared in gsl_blas.h doesn't allow COLUMN-major matrix storage option*/
#include <gsl/gsl_cblas.h>
#include <gsl/gsl_math.h>


double MT_kernel(double * v_MH_mean, double * MKL_T_InvCov, double MT_mu, int N, double * THETA_MH_i1);

double MT_kernel(double * v_MH_mean, double * MKL_T_InvCov, double MT_mu, int N, double * THETA_MH_i1)
{
	int i;
	double *demeaned = new double [N];
	double *temp1 = new double [N];
	double kernelm[1];
	double ans;

for (i=0;i<N;i++)
	demeaned[i] = THETA_MH_i1[i] - v_MH_mean[i];

//******temp1 = (demeaned)' * MKL_T_InvCov
cblas_dgemm (CblasColMajor, CblasTrans, CblasNoTrans, 1, 17, 17, 1.0, 
			 demeaned, 17, MKL_T_InvCov, 17, 0.0, temp1, 1);

//******kernelm = temp1 * demeaned
cblas_dgemm (CblasColMajor, CblasNoTrans, CblasNoTrans, 1, 1, 17, 1.0, 
			 temp1, 1, demeaned, 17, 0.0, kernelm, 1);

ans = pow((1+kernelm[0]/MT_mu),(-(MT_mu+N)/2));

delete [] demeaned;
delete [] temp1;

return ans;

}