#include <stdio.h> 
#include <memory.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>

//#include "mkl.h"
/*Note that blas_dgemm declared in gsl_blas.h doesn't allow COLUMN-major matrix storage option*/
#include <gsl/gsl_cblas.h> //Note that MKL and GSL libraries interfere here.
#include <gsl/gsl_math.h>
//#include <gsl/gsl_vector.h>
//#include <gsl/gsl_matrix.h>
#include <gsl/gsl_linalg.h>
#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>
 

#include"dfridders.h"

#define lags 20 //Maximum number of lags for ACF series

//Defining the PARAMETER structure type
typedef struct
{
	double theta_tau;
	double theta_beta;
	double theta_k;
	double theta_psi1;
	double theta_psi2;

	double theta_rho_pi;
	double theta_rho_r;
	double theta_rho_g;
	double theta_rho_z;

	double theta_D;

	double theta_pi_star;
	double theta_r_star;

	double theta_sigma_r;
	double theta_sigma_z;
	double theta_sigma_g;
	double theta_sigma_pi;
	double theta_rho_gz;
} dsge_param;

// Defining the structure type that will be used to construct data
typedef struct
{
    int data_date;           /* quarter number */
    double data_r;           /* target interest rate */
    double data_pi;          /* inflation rate  */
    double data_gdp;         /* HP-filtered GDP */
    double data_delta;       /* discreteness adjustment */

} dsge_data_obj;

struct DSGE_like_params 
{ 
	dsge_data_obj *data_X;
	int data_T; 
	gsl_vector *v_x;
	int dim;
	int partial;
};

extern gsl_rng * r; //Reference to global seed r defined in DSGE.c

// Matrix form conversion function from MKL to GSL of size (GSL_row x GSL_col)
void MKLtoGSL (double MKL_mat[], gsl_matrix * GSL_mat, int GSL_row, int GSL_col);

// Matrix form conversion function from GSL of size (GSL_row x GSL_col) to MKL
void GSLtoMKL (gsl_matrix * GSL_mat, double MKL_mat[], int GSL_row, int GSL_col);

// Function for printing GSL matrix to screen
void print_scr_gsl_mat (gsl_matrix * GSL_mat, int r, int c);

// Function for printing a GSL vector to screen
void print_scr_gsl_vec (gsl_vector * GSL_vec, int c);

// Function for printing MKL matrix to screen
void print_scr_mkl_mat (double MKL_mat[], int r, int c);

// Function for setting the first n elements of an array a equal to the first n elements of array b
void eqarrays(double a[], double b[], int n);

extern double lglike_fn(dsge_param *THETA_par, dsge_data_obj *data_X_par, int MH_i_par, int data_T_par);

//Note that this function produces a NEGATIVE of likelihood. So, the resulting Hessian is a negative Hessian!
extern double DSGE_like_f (const gsl_vector *v_THETA, void *p);
int hessian( const gsl_vector* x, double (*f)( const gsl_vector*, void* ), void* params, double h, gsl_matrix* H );

//Draw from N-dimensional Student-t
void MT_sampler(double * v_MH_mean, double * MKL_Chol_Cov, double scale, double MT_mu, int N, gsl_rng * r, double *mult_t);
void MN_sampler(double *v_MH_mean, double *MKL_Chol_Cov, 
				double scale,  int N, gsl_rng *r, double *mult_norm);

// Calculates Kernel of Multivariate Student-T pdf
double MT_kernel(double * v_MH_mean, double * MKL_T_InvCov, double MT_mu, int N, double * THETA_MH_i);

//AutoCorrelation Function for M-H diagnostics
extern double *autocorr(double *data,int n,int l);

// Function that fills in the KF implied i_par(th) state values for all times given MH_i_par draw
void state_average(dsge_param *THETA_par, dsge_data_obj *data_X_par, int MH_i_par, int data_T_par, gsl_matrix * GSL_states, int i_par, int i_burnin);

extern double h; //Reference to global derivative step size h defined on the next line

void MH_Tailoring(DSGE_like_params * param, dsge_param *THETA, gsl_vector *v_THETA_MLE, int MH_T, unsigned int Tailor);

void MH_Tailoring(DSGE_like_params * param, dsge_param *THETA, gsl_vector *v_THETA_MLE, int MH_T, unsigned int Tailor)

{
	int i,j;//various indecies
	int MH_i; //MH iteration loop index variable
	double MH_alpha; //M-H acceptance probability
	double AR_alpha; //A-R acceptance probability
	double MH_u; //used in M-H uniform draw
	double accept_rate=0.0; //Acceptnce rate in the M-H algorithm Overall
	//double AR_accept_rate=0.0; //Acceptnce rate in the A-R part of the M-H algorithm
	double accept_indicat; //Indicator of Acceptance variable
	double AR_counter = 0.0;//Counter of A-R iterations
	double burnin = 0.1; //Burn-in rate in M-H algorithm (proportion of discarded draws in the beginning)
	double ACF[lags][17];//Pointer to AutoCorr Function Structure
	int i_burnin = (int) (MH_T*burnin); //Integer number of burnin draws
	double *MH_par = new double [MH_T - i_burnin]; //Uses in ACF calculation 
	double RW_scale=1.0;//scale for the "square root" of the covariance matrix
	
//M-H proposal density variables:
	double v_MH_mean[17]; //Vector of proposal density's mean
	double MT_mu = 15.0; // Multivariate T degrees of freedom
	gsl_matrix * m_Cov = gsl_matrix_alloc(17,17); //Covariance Matrix for proposal density
	double MKL_T_InvCov[17*17]; //INVERSE of m_Cov only in MKL form
	double MKL_Chol_Cov[17*17]; //"transpose(sqrt(m_Cov))" by Cholesky decomposition
	double THETA_MH_i0[17]; //Array needed to feed to MT_kernel to calculate qold
	double THETA_MH_i1[17]; //Draw from the proposal density

//A-R variables
	double AR_c = 1.5; /*tuning constant in the Accept-Reject step*/
	double temp1, temp2, temp3;/*used as temporary holders*/
	double pold, qold, pnew, qnew; /*used in MH step of A-R Tailoring to determine aplha*/
	int C1, C2;

  	clock_t start1=0,stop1=0,stop2=0,stop3=0; //Speed timing variables
	double Time=0;

	dsge_data_obj *data_X;
	int data_T;
	data_X = (*param).data_X;
	data_T = (*param).data_T;

	gsl_matrix * GSL_states = gsl_matrix_alloc ((MH_T-i_burnin), data_T); 

	gsl_matrix * negHessian = gsl_matrix_alloc(17,17);
	gsl_permutation * permut = gsl_permutation_alloc (17);
	int signum;

//C++ Hessian/Gradient Code - related variables
	gsl_vector * g,* error;
  struct Dfridders* Df = get_dfridders( 0 );
  g = gsl_vector_alloc( 17 );
  error = gsl_vector_alloc( 17 );

	for (i=0;i<17;i++)
	{
/*Setting the Proposal Density Mean vector
 v_MH_mean[i] = *(&THETA[0].theta_tau+i);*/
v_MH_mean[i] = gsl_vector_get(v_THETA_MLE,i);
/*Setting to zero the sum used to calculate the mean of MH draws*/
*(&THETA[MH_T+1].theta_tau+i) = 0.0;
/*setting the first elements of THETA equal to MLE
  (it will be overwritten at the first iteration anyway,
  but I need it only before that to calculate AR_c*/
*(&THETA[1].theta_tau+i) = gsl_vector_get(v_THETA_MLE,i);
	}

//NEGATIVE Hessian of likelihood:
hessian( v_THETA_MLE, &DSGE_like_f, param, h, negHessian );
//both hessian( v_THETA_MLE, &DSGE_like_f,... AND hessian( v_THETA_MLE, DSGE_like_f,...
//will work and produce the same result (see email from John Lamb for explanation)
															 
printf("Negative Hessian: \n");
print_scr_gsl_mat (negHessian, 17, 17);

dfridders( Df, g, error, &DSGE_like_f, v_THETA_MLE, param, h );
printf("\n Gradient Using C++ code from John Lamb: \n");
print_scr_gsl_vec (g,17);
printf("Gradient Errors \n");
print_scr_gsl_vec (error,17);
  gsl_vector_free( error );
  gsl_vector_free( g );

/*retaining the INVERSE of covariance Matrix of Tailored Proposal Density
  to be used later in the Mult T kernel function*/
GSLtoMKL(negHessian, MKL_T_InvCov, 17, 17);


/*Inverting the negative Hessian to get the covariance matrix for proposal*/
gsl_linalg_LU_decomp (negHessian, permut, &signum);
gsl_linalg_LU_invert (negHessian, permut, m_Cov);

printf("Covariance Matrix of Tailored Proposal Density: \n");
print_scr_gsl_mat (m_Cov, 17, 17);
printf("Mean of Tailored Proposal Density: \n");
print_scr_mkl_mat(v_MH_mean, 17, 1);

gsl_matrix_free (negHessian);
gsl_permutation_free (permut);
//************Preparing Hessian matrix for M-H algorithm****************************************


gsl_linalg_cholesky_decomp(m_Cov);//this stupid routine on output puts in the diagonal and lower triangular part of the input
								  //matrix m_Cov the matrix L and in the upper triangular part L^T, so:
for (i=0;i<16;i++)
{
	for(j=i+1;j<17;j++)
	gsl_matrix_set(m_Cov,i,j,0.0);
}
GSLtoMKL (m_Cov, MKL_Chol_Cov, 17, 17);
gsl_matrix_free (m_Cov);


/*********************The main MH iteration loop will begin here.********************************/
	start1=clock();
for (MH_i= 0;MH_i<MH_T;MH_i++)
{
			/*Drawing from N-dim Multivariate Student-t proposal*/
			MT_sampler(v_MH_mean, MKL_Chol_Cov, RW_scale, MT_mu, 17, r, THETA_MH_i1);
			for (i=0;i<17;i++)
				*(&THETA[MH_i+1].theta_tau+i) = THETA_MH_i1[i];
	
} //The end of the main MH iteration loop ********************************************************************

/*Output Mult_T to a file*/
	{
	int i_f, MH_i_f;
	FILE * f_multt;
 	f_multt = fopen("Mult_T.txt", "w");

	for (MH_i_f= 0;MH_i_f<MH_T;MH_i_f++)
		{
		for (i_f=0;i_f<17;i_f++)
		fprintf (f_multt, "%.8g   ", *(&THETA[MH_i_f].theta_tau+i_f) );
		fprintf (f_multt, "\n");
		}
	fclose (f_multt);
	}  
}