// DSGE.cpp : Defines the entry point for the console application.
//

#include <stdio.h>
#include <math.h>
#include <time.h>
#include <stdlib.h>

//#include <gsl/gsl_blas.h> //Note that MKL and GSL libraries interfere here.
#include <gsl/gsl_math.h>
#include <gsl/gsl_vector.h>
#include <gsl/gsl_matrix.h>

// Matrix form conversion function from MKL to GSL of size (GSL_row x GSL_col)
void MKLtoGSL (double MKL_mat[], gsl_matrix * GSL_mat, int GSL_row, int GSL_col);

// Matrix form conversion function from GSL of size (GSL_row x GSL_col) to MKL
void GSLtoMKL (gsl_matrix * GSL_mat, double MKL_mat[], int GSL_row, int GSL_col);

// Function for printing GSL matrix to screen
void print_scr_gsl_mat (gsl_matrix * GSL_mat, int r, int c);

// Function for printing a GSL vector to screen
void print_scr_gsl_vec (gsl_vector * GSL_vec, int c);

// Function for printing MKL matrix to screen
void print_scr_mkl_mat (double MKL_mat[], int r, int c);

// Function for setting the first n elements of an array a equal to the first n elements of array b
void eqarrays(double a[], double b[], int n);





/*function for converting GSL_row*GSL_col MKL_mat recorded as a 1-dimensional column-major FORTRAN-style array
into a rectangular GSL_row x GSL_col GSL-format matrix GSL_mat*/
void MKLtoGSL (double MKL_mat[], gsl_matrix * GSL_mat, int GSL_row, int GSL_col)
{
	int i,r,c;
	
	for(i=0;i<GSL_row*GSL_col;i++)

	{	//(int) () takes the integer component of any number
		//note that MKL_mat[] matrix is in COLUMN-MAJOR Fortran-like form
		c = (int)(i/GSL_row) /*column*/; r = (int)(i-c*GSL_row); /*row*/
		gsl_matrix_set(GSL_mat,r,c,MKL_mat[i]);
	}

}


/*function for converting a square GSL_size x GSL_size GSL-format matrix GSL_mat into GSL_size^2 MKL_mat
recorded as a 1-dimensional column-major FORTRAN-style array*/
void GSLtoMKL (gsl_matrix * GSL_mat, double MKL_mat[], int GSL_row, int GSL_col)
{
	int i,j,arrayind;
	
	for(i=0;i<GSL_col;i++)
	{
	for(j=0;j<GSL_row;j++)
		{	
		arrayind = i*GSL_row +j;
		MKL_mat[arrayind] = gsl_matrix_get(GSL_mat,j,i);
		}
	}
}


// Function for printing a GSL rectangular matrix to screen
void print_scr_gsl_mat (gsl_matrix * GSL_mat, int r, int c)
{
	int i,j;
	for (i = 0; i < r; i++)
	{
		for (j = 0; j < c; j++)
		{
	/*set all small elements to zero (they are probably are zeros anyway, 
	only due to some numerical approximation they are a little different from zero)*/
		if(abs(gsl_matrix_get (GSL_mat, i, j)) < 0.000000000000001)
			gsl_matrix_set (GSL_mat, i, j,0.0);
		printf("GSL(%d,%d) = %g    ", i, j, gsl_matrix_get (GSL_mat, i, j));
		}
	printf("\n");
	}
printf("\n");
}

// Function for printing a GSL vector to screen
void print_scr_gsl_vec (gsl_vector * GSL_vec, int c)
{
	int i;
	for (i = 0; i < c; i++)
	{
	/*set all small elements to zero (they are probably are zeros anyway, 
	only due to some numerical approximation they are a little different from zero)*/
		if(abs(gsl_vector_get (GSL_vec, i)) < 0.000000000000001)
			gsl_vector_set (GSL_vec, i,0.0);
		printf("GSL(%d) = %.8g \n", i, gsl_vector_get (GSL_vec, i));
	}
printf("\n");
}

//function for printing MKL matrix to a screen
void print_scr_mkl_mat (double MKL_mat[], int r, int c)
{
	int i,j;
	for (i = 0; i < r; i++)
	{
		for (j = 0; j < c; j++)
		{
	/*set all small elements to zero (they are probably are zeros anyway, 
	only due to some numerical approximation they are a little different from zero)*/
		if(abs(MKL_mat[j*r+i]) < 0.000000000000001)
			MKL_mat[j*r+i]=0.0;
			printf("MKL(%d,%d) = %g    ", i, j, MKL_mat[j*r+i]);
		}
		printf("\n");
	}
printf("\n");
}


// Function for setting the first n elements of an array a equal to the first n elements of array b
void eqarrays(double a[], double b[], int n)
{
	int i;
	for (i = 0; i < n; i++)
	{
		a[i]=b[i];
	}
}

