#include<gsl/gsl_vector.h>
#include<gsl/gsl_matrix.h>

#ifndef DFRIDDERS_H
#define DFRIDDERS_H

#ifdef __cplusplus
extern "C" {
#endif

  /**
   * This struct holds data that can be initialised and finalised 'on the fly'
   * for dfridders
   */
  struct Dfridders;
  
  /**
   * Actually estimate the derivative using Ridders' method.
   * @param df A struct Dfridders. Use get_dfridders to get this value.
   * @param g The gradient estimate.
   * @param error Its error estimate.
   * @param function A multivariate function.
   * @param x The vector used by function.
   * @param params Any other parameters you want to pass to the function.
   * @param h A small value.
   * @return 0 for success, GSL error codes on failure.
   */
  int dfridders( const struct Dfridders* df,
		 gsl_vector* g,
		 gsl_vector* error,
		 double (*function)( const gsl_vector*, void* ),
		 const gsl_vector* x,
		 void* params,
		 const double h );

  /**
   * A function to access one of three struct Dfridders objects. A Dfridders object
   * contains data that has to be allocated at runtime and destroyed at the end of
   * the program. C cannot handle this without explicit malloc and free calls and so
   * we use C++. This function allows us access to pointers to C++ objects withou us
   * having to know the size explicitly. There are three objects by default. This is
   * so that calls to Dfridders can be made efficiently for up to three functions
   * with different gsl_vector sizes. Since Dfridders must initialise for an
   * vector size that may be unknown until run time, it is most efficient to allocate
   * memory only once per object. If an object is used with vectors of different
   * sizes, reallocation would be needed, which is undesirable in numerical code.
   * Hence, three objects are available, making the need for reallocation less
   * likely.
   * @param i An interger: use 0, 1 or 2.
   * @return An object that can be used with dfridders.
   */
  struct Dfridders* get_dfridders( int i );

#ifdef __cplusplus
}
#endif

#endif
